unit UCalculate;

interface

uses SysUtils, UmainForm, UDataMod;

var
  WarnFile, TransFile,
  OutFile, FrqFile            : textfile;
  NUM, XX, YY, YI             : integer;
  Flag, WTdepth, Busy, Almost,
  Done, WarnedKy, Fixed,
  WarnedInit, WarnedRota      : boolean;
  Warn1, Warn2                : array [1..4] of boolean;
  EPU0, U, IRAI, IRBI         : array [1..4] of real;
  AC, BC, UC, DA, DB, DU,
  CAA, CAB, CAU, CAT, CIT,
  DRA, DRB, DWT, DWTI, DWT0,
  EFA, EFB, EFT,
  EPAC, EPBC, EPUC,
  SCRA, SCRB, SCRU,
  SCRAM, SCRBM, SCRZ,
  SCtrA, SCtrB, SCtr, SCaq, SALT,
  PLA, PLB, PLT, PLU, SUA, SUB,
  ROP, TRP, AQP, ZDR, ZWL      : real;
  CiNa, ROTA0, ROTAI           : integer;
//for conversions
  AA, BB, AI, BI, BYP, DRR, DS,
  FF,  HIG, HOG, IRA, IRB, RAI,
  PLC, {SiU, SoU,} SDA, SDB,
  SRI, SRO, STA, STB, STU,
  UU, WEL, WTC,  WEL0, DRR0       : array [1..4] of real;
  CIR, CDW, DRC, ZIR,
  DIFP, DIFC,
  SCRA0, SCRB0, SCRU0, SCRZ0,
  ZIC, Zigw, Zrain, ZIR0          : real;
//Transfer values
  DWTp, SCRAp, SCRBp, SCRUp,
  SCtrAp, SCtrBp, SCtrp, SCaqp,
  SALTp, ACp, BCp, UCp, DDp           : real;
  FIRSTp, KDp, KYp, YIp        : byte;
  PermanentFile                : string;

  procedure Calculations;
  procedure InitSettings;
  procedure Conversions;                               {for old Fortran symbols}
  procedure Opening;                        {open output files, writing headers}
  procedure Transfer;            {transfer data for annual calculations if Ky=1}
  procedure AreaFractions;
  procedure InitSal;  {intital settings of soil salinity depending on rotation}
  procedure WriteOutputData (var AA,BB,BYP,DS,DRR,IRA,IRB,PLC,UU,FF,WEL : real);
  procedure TransValues;                              {part of WriteOutputData}
  procedure Frequency (var AA,BB,UU,DS:real; var YR:integer);

  procedure WaterBalances (var AAy,BBy,BYPy,DRRy,DSy,EPAy,EPBy,EPUy,FFy,
            HIGy,HOGy,IRAy,IRBy,PLCy,RAIy,RCAy,RCBy,SDAy,SDBy,SRIy,SROy,
            STAy,STBy,STUy,UUy,WELy,WTCy : real);
  procedure AuxWBA (var AA1,BB1,EPA1,EPB1,EPU1,IRA1,IRB1,RAI1,SDA1,SDB1,
            SRI1,SRO1,STA1,STB1,STU1,UU1,ZZ1 : real);      {under WaterBalances}
  procedure AuxWBA1 (var CA,EP0,EP1,IR,RAIN,SD,ST,ZZ2 : real);    {under AuxWBA}

  procedure SaltBalances (var AAy,BBy,DRRy,FFy,HIGy,HOGy,IRAy,IRBy,PLCy,RAIy,
            SDAy,SDBy,SRIy,SROy,UUy,WELy : real);
  procedure AuxSB1 (var DOWNx,UPWAx,INFAx,INFBx,INFUx,RAINx,SCMAx,SCMBx,SCMUx,
                    SCXx,SDAx,SDBx,SDLAx,SDLBx,SDLUx,SROx,SSIx : real);
  procedure AuxSB2 (var AUB,SCRp,SCMp,SCOp,SCRq,SCMq,SCOq,TSCp,TSCq,TSIp,TSIq,
            TPLp,TPLq: real);                               {under SaltBalances}
  procedure AuxSB3 (var PLF,SCRAx,SCMAx,TSIx : real);       {under SaltBalances}
  procedure AuxSB4A (var PLC,SCtrMx,TSPx,VDDx,VUSx : real);  {under SaltBalances}
  procedure AuxSB4B (var PLC,{SCaqx,SCtrBx,}SCtrBMx,{SCtrAx,}SCtrAMx,TSPx,
            VDDx,VUSx : real);                              {under SaltBalances}
  procedure AuxSB5 (var HIG,HOG,PLC,PLF,SCtrBMx,SCTMx,VDDx,VUSx,WELx,
            Zigwx,ZIRx : real);                              {under SaltBalances}

  procedure Efficiency (var AAy,BBy,EPAy,EPBy,IRAy,IRBy,RAIy : real);
  procedure FarmResponse (var AAy,A0,BBy,B0,BYPy,IRAy,IRBy,
            STAy,STBy : real);
  procedure ReduceIRR (var AAy,BBy,BYPy,IRAy,IRAIy,IRBy,IRBIy,
            RCAy,RCBy : real);


implementation

procedure Calculations;
{---------------------}
label 1, 2;
var I, II, J, K, M, N, NrOfErrorLines : integer;
    RunError:boolean;
    ACCUMA, ACCUMU : real;
begin with DataMod do
begin
//Inital settings
  InitSettings;                          {IF Ky=1 then this happens every year}
//Temporary conversions using old Fortran symbols,
//later to be replaced by DataMod symbols
  Conversions;
//Opening Warning file, opening/reading a transfer file if required,
  Opening;
//transfer data for annual calculations if Ky=1
  Transfer;
//Calculating the unirrigated area fraction (UU) and the smallest area
//fractions per year (AC,BC,UC), changing the value of Kr if required
  AreaFractions;
//Setting initial values of soil salinities depending on Kr
  InitSal;
//Writing initial data to the output file and frequency file
  IF YI=0 THEN          {with annnual calculations YI=0 only in the first year}
  begin
    FOR J:=1 to Ns do
    begin
      YY:=J;
      EPU0[J]:=EpU[J];            {initial evaporation from unirrigated U land}
//Writing initial data of year 0 to *.RES and *.FRQ
      WriteOutputData (AA[J],BB[J],BYP[J],DS[J],DRR[j],IRA[J],IRB[J],PLC[J],
                       UU[J],FF[j],WEL[J]);
    end;
  end; {IF YI=0 THEN}

//Calculations for N years
  N:=NS*NUM;
  FOR M:=1 to N DO
  BEGIN
    XX:=M;
//Setting the value of the season indicator YY
    for K:=1 to NS do
    begin
      I:=K;
      while I<NS*NUM+1 do
      begin
        if I=XX then
        begin
          YY:=K;
          goto 1;
        end;
        I:=I+NS;
      end;
    end;

//Calculating the unirrigated area fraction (UU) and the smallest area
//fractions per year (AC,BC,UC), changing the value of Kr if required
1:  IF YY=1 THEN AreaFractions;
    if not Busy and (XX>NS*NUM div 3) then
    begin
      MainForm.WaitMemo.Lines.Add(' Still busy .....');
      sleep (500);
      Busy:=true;
    end;
    if not Almost and (XX>2*NS*NUM div 3) then
    begin
      MainForm.WaitMemo.Lines.Add(' Almost done ....');
      sleep (500);
      Almost:=true;
    end;

//Determining water balances and a new depth of the watertable through
//procedure Waterbalances with secondary subroutines
    BYP[YY]:=Io[YY];
    if AA[YY]+BB[YY]>0 then DRR[YY]:=DRR0[YY]/(AA[YY]+BB[YY])
    else
       DRR[YY]:=0;
    if AA[YY]+BB[YY]>0 then WEL[YY]:=WEL0[YY]/(AA[YY]+BB[YY])
    else
       WEL[YY]:=0;
    if (AA[YY]+BB[YY])>0 then
       IRA[YY]:=IRAI[YY]+AA[YY]*(DRR0[YY]+FF[YY]*WEL0[YY])/(AA[YY]+BB[YY])
    else
       IRA[yy]:=0;
    if (AA[YY]+BB[YY])>0 then
       IRB[YY]:=IRBI[YY]+BB[YY]*(DRR0[YY]+FF[YY]*WEL0[YY])/(AA[YY]+BB[YY])
    else
       IRB[YY]:=0;
    PLC[YY]:=Lc[YY];
    DWT0:=DWT;
    if (SCRU>16) then EPU[YY]:=4*EPU0[YY]/sqrt(SCRU);
    Waterbalances (AA[YY],BB[YY],BYP[YY],DRR[YY],DS[YY],
                  EPA[YY],EPB[YY],EPU[YY],FF[YY],HIG[YY],HOG[YY],
                  IRA[YY],IRB[YY],PLC[YY],RAI[YY],RCA[YY],RCB[YY],
                  SDA[YY],SDB[YY],SRI[YY],SRO[YY],STA[YY],STB[YY],
                  STU[YY],UU[YY],WEL[YY],WTC[YY]);

//Only for checking
   CAT:=AA[YY]*CAA+BB[YY]*CAB+UU[YY]*CAU;

  //Salt balances and new salinities with secondary subroutines
    SCRA0:=SCRA;
    SCRB0:=SCRB;
    SCRU0:=SCRU;
    SCRZ0:=SCRZ;
    SaltBalances (AA[YY],BB[YY],DRR[YY],FF[YY],HIG[YY],HOG[YY],
                  IRA[YY],IRB[YY],PLC[YY],RAI[YY],SDA[YY],SDB[YY],
                  SRI[YY],SRO[YY],UU[YY],WEL[YY]);

//The following is only for a calculation check
    ACCUMA:=IRA[YY]*ZIR+CAA*SCtr*FLx-PLA*SCRA*FLR;
    ACCUMU:=CAU*SCtr*FLx-PLU*SCRU*FLR;

//Determining field irrigation efficiencies, canal irrigation
//efficiencies, and sufficiencies of irrigation water
    Efficiency (AA[YY],BB[YY],EPA[YY],EPB[YY],IRA[YY],IRB[YY],RAI[YY]);

//Writing output data to the output file, calculating and writing
//frequency distributions through subroutine Frequency
    WriteOutputData (AA[YY],BB[YY],BYP[YY],DS[YY],DRR[YY],IRA[YY],IRB[YY],
                     PLC[YY],UU[YY],FF[YY],WEL[YY]);

    IF (YY=NS) and (DWT>Dr+Dx+Dq) THEN			                {at end of year}
    BEGIN
      WTDepth:=true;
      if not AnnualCalc then
      begin
        II:=YI+(XX+NS-1) div NS;
        WRITELN (Warnfile);
        WRITELN (WarnFile,' The watertable drops below bottom of the aquifer'+
                          ' in year:',II:3);
        WRITELN (WarnFile,' Calculations are interrupted. Please revise the'+
                          ' input data.');
        WRITELN (Warnfile);
        GOTO 2;
      end;
    END;

//Decreasing the seasonally irrigated area fractions when no reclamation
//takes place (Kf=1) and salinization or insufficient irrigation
//occurs; increasing the irrigation intensities, and decreasing
//bypass when required; the reverse when the salinity reduces again
    IF Kf=1 THEN
       FarmResponse (AA[YY],AI[YY],BB[YY],BI[YY],BYP[YY],
                     IRA[YY],IRB[YY],STA[YY],STB[YY]);
//Reducing irrigation stepwise when the water table rises above the soil surface
  if (DWT<0.35) then
  begin
    ReduceIrr (AA[YY],BB[YY],BYP[YY],IRA[YY],IRAI[YY],IRB[YY],IRBI[YY],
               RCA[YY],RCB[YY]);
    IRAI[YY]:=IRA[YY];
    IRBI[YY]:=IRB[YY];
    IaA[YY]:=IRA[YY];
    if IaA[YY]<0.001 then IaA[YY]:=0;
    IaB[YY]:=IRB[YY];
    if IaB[YY]<0.001 then IaB[YY]:=0;
    Io[YY]:=BYP[YY];
    if not Done and ((IaA[YY]>0) or (IaB[YY]>0)) then
    begin
      WRITELN (Warnfile);
      writeln (WarnFile,'    As the water table rises to the soil surface,');
      writeln (WarnFile,'    the irrigation is reduced.');
      WRITELN (Warnfile);
      Done:=true;
    end;
  end;

//Warning about pumping from wells when the watertable drops
    if WEL[YY]>0.1 then
    begin
      if (DWTI<5) and (DWT>10) THEN
      begin
        WEL[YY]:=0.9*WEL[yy];
        WEL0[YY]:=WEL[YY];
        if not WarnedKy then
        begin
          WRITELN (Warnfile);
          writeln (WarnFile,'   As the water table drops considerably,');
          writeln (WarnFile,'   the pumping from wells needs to be reduced.');
          WRITELN (Warnfile);
          WarnedKy:=true;
        end;
      end;
    end;

    IF (Kf=1) AND (YY=NS) THEN
//Calculating the unirrigated area fraction (UU) and the smallest area
//fractions per year (AC,BC,UC), changing the value of Kr if required
    BEGIN
      ACP:=AC;
      BCP:=BC;
      UCP:=UC;
      AreaFractions;
      DA:=AC-ACP;
      DB:=BC-BCP;
      DU:=UC-UCP;
      IF Kr=1 THEN
      BEGIN
        IF DU>0 THEN SCRU:=(UCP*SCRU+DU*SCRA)/(UCP+DU);
        IF DA>0 THEN SCRA:=(ACP*SCRA+DA*SCRU)/(ACP+DA);
      END;
      IF Kr=2 THEN
      BEGIN
        IF DA>0 THEN SCRA:=(ACP*SCRA+DB*SCRB)/(ACP+DB);
        IF DB>0 THEN SCRB:=(BCP*SCRB+DB*SCRA)/(BCP+DB);
      END;
      IF Kr=3 THEN
      BEGIN
        IF DB>0 THEN SCRB:=(BCP*SCRB+DB*SCRA)/(BCP+DB);
        IF DA>0 THEN SCRA:=(ACP*SCRA+DA*SCRB)/(ACP+DA);
      END;
    END; {IF (Kf=1) AND (YY=NS) THEN}

//Continuing the calculations for the next season
    DWT0:=DWT;                                   {new initial watertable depth}

  END; {FOR XX=1 to NS*NUM DO}                            {end of calculations}

  IF (Ky<>1) OR (FIRST=2) THEN                    {if not annual calculations}
  BEGIN
    FIRST := 2;
    FileName := ChangeFileExt(FileName,'.trf');
    Assignfile(TransFile, FileName);
    Rewrite (TransFile);
    WRITELN (TransFile,FIRST);
    closefile (TransFile);
  END;

2:closefile (OutFile);
  closefile (FrqFile);
  if (Ky=0) then closefile (WarnFile);

end; {with DataMod do}
end; {procedure Calculations}
{---------------------------}


procedure InitSettings;
{---------------------}
var I:byte;

begin with DataMod do
begin
  Flag:=false;
  Busy:=false;
  Almost:=false;
  DWTI:=Dw0;
  DWT0:=Dw0;
  XX:=0;
  SALT:=0;
  ZIR0:=0;
  NUM:=Ny;
  For I:=1 to Ns do
  begin
    AI[I]:=A[I];                                               {initial A area}
    BI[I]:=B[I];                                               {initial B area}
    IRAI[I]:=IaA[I];                             {initial irrigation of A area}
    IrBI[I]:=IaB[I];                             {initial irrigation of A area}
    WEL0[I]:=Gw[I];                                {initial pumping from wells}
    DRR0[I]:=Gu[I];                                {initial drainage reuse}
  end;

end; {with DataMod do}
end; {procedure InitSettings}
{---------------------------}


//Temporary conversions using old Fortran symbols,
//later to be replaced by DataMod symbols
procedure Conversions;
{-------------------}
var ii : byte;

begin with DataMod do
begin

//reals
  SCRAM:=CAmax;
  SCRBM:=CBmax;
  CDW:=Dc;
  DWT:=Dw0;
  SCaq:=Cq0;
  SCtrB:=CxB0;
  SCtr:=Cx0;
  SCtrA:=CxA0;
  ZIC:=Cic;
  Zigw:=Ch;
  Zrain:=Cp;
//real arrays
  for ii:=1 to Ns do
  begin
    AA[ii]:=A[ii];
    BB[ii]:=B[ii];
    BYP[ii]:=Io[ii];
    DS[ii]:=Ts[ii];
    DRR[ii]:=Gu[ii];
    FF[ii]:=Fw[ii];
    HIG[ii]:=Gi[ii];
    HOG[ii]:=Go[ii];
    IRA[ii]:=IaA[ii];
    IRB[ii]:=IaB[ii];
    RAI[ii]:=Pp[ii];
    PLC[ii]:=Lc[ii];
    SDA[ii]:=SoA[ii];
    SDB[ii]:=SoB[ii];
    SRI[ii]:=SiU[ii];
    SRO[ii]:=SoU[ii];
    STA[ii]:=FsA[ii];
    STB[ii]:=FsB[ii];
    STu[ii]:=FsU[ii];
    UU[ii]:=U[ii];
    WEL[ii]:=Gw[ii];
    WTC[ii]:=Frd[ii];
  end;

end; {with DataMod do}
end; {procedure Conversions}
{--------------------------}


//To open warning, output (*.Out) and *.Frq files, write headers
procedure Opening;
{----------------}
var AuxFile : string;
begin with DataMod do
begin

//Opening warning file
  Assignfile(WarnFile, 'Warning.lst');
  rewrite (WarnFile);
  if not WarnedInit then
  begin
    writeln (WarnFile,' THE FOLLOWING RUNTIME WARNINGS ARE GIVEN:');
    WarnedInit:=true;
  end;
//Opening the *.RES output file
  FileName := ChangeFileExt(FileName,'.res');
  if Ky=1 then FileName := ChangeFileExt(PermanentFile,'.res');
  Assignfile (OutFile, FileName);
  IF (Ky=0) OR ((Ky=1) AND (YI=0)) THEN
     Rewrite (OutFile);
  IF (Ky=1) AND (YI>0) THEN
     Append(OutFile);
//Opening the *.FRQ output file
  FileName := ChangeFileExt(FileName,'.frq');
  if Ky=1 then FileName := ChangeFileExt(PermanentFile,'.frq');
  Assignfile (FrqFile, FileName);
  IF (Ky=0) OR ((Ky=1) AND (YI=0)) THEN
      Rewrite (FrqFile);
  IF (Ky=1) AND (YI>0) THEN
     Append(FrqFile);

//Initially writing a header in output file
  IF YI=0 THEN
  begin
    writeln (OutFile, ' SALTMOD: A predictive computation method for'+
                      ' soil and groundwater,');
    writeln (OutFile, ' salinity and the watertable depth in'+
                      ' agricultural lands using');
    writeln (OutFile, ' varying hydrologic conditions and'+
                      ' watermanagement options.');
    writeln (OutFile, ' The first program was developed in Fortran by'+
                      ' R.J.Oosterbaan and');
    writeln (OutFile, ' Isabel de Lima at ILRI, Wageningen The Netherlands.');
    writeln (OutFile, ' The first user menu in TurboPscal was developed'+
                      ' by Henk Ramnandanlal');
    writeln (OutFile, ' and Rob Kselik.');
    writeln (OutFile, ' The first Windows version 1.0 of August 2006 was made'+
                      ' by Oosterbaan,');
    writeln (OutFile, ' and updated regularly.');
    writeln (OutFile);
    AuxFile:=changefileext (FileName,'.RES');
    writeln (OutFile, ' Name of this output (result) file: ',AuxFile);
    AuxFile:=changefileext (FileName,'.INP');
    writeln (OutFile, ' Name of input (data) file used   : ',AuxFile);
    if (Kf=1) and (Kw=1) then
        writeln (OutFile,' All farmers responses are on.');
    if (Kf=1) and (Kw=0) then
        writeln (OutFile,' Farmers responses are on, but not for'+
                         ' rotation type.');
    if (Kf=0) then
        writeln (OutFile,' Farmers responses are off.');
    if (Ky=1) then
        writeln (OutFile,' Calculations were done year by year.')
    else
        writeln (OutFile,' Calculations were done for all years in one go.');
  end; {IF YI=0 THEN}

end; {with DataMod do}
end; {procedure Opening}
{----------------------}


procedure Transfer; {Using transfer data if needed (Ky=1, annual calculations)}
{-----------------}
begin with DataMod do
begin
  IF Ky=1 THEN
  BEGIN
    NUM:=1;
    Kf:=0;
    if FIRST=1 then
       FIRST:=0
    else
    begin
//Resetting transfer values;
      FIRST:=FIRSTp;
      DWT:=DWTp;
      SCRA:=SCRAp;
      SCRB:=SCRBp;
      SCRU:=SCRUp;
      SCtrA:=SCtrAp;
      SCtrB:=SCtrBp;
      SCtr:=SCtrp;
      SCaq:=SCaqp;
      SALT:=SALTp;
      AC:=ACp;
      BC:=BCp;
      UC:=UCp;
      Dd:=DDp;
      Kd:=KDp;
      YI:=YIp;
      Ky:=KYp;
//Checking change of subsurface drainage system
      IF(KDp=0) AND (Kd=1) THEN
      begin
//        SCtrA:=Cxa0;
//        SCtrB:=Cxb0;
      end;
      IF (KDp=1) AND (Kd=1) THEN
         IF (Dd>=DDp) THEN
             SCtrA:=((DDp-Dr)*SCtrA+(Dd-DDp)*SCtrB)/(Dd-Dr)
         ELSE
             SCtrB:=((Dx-DDp)*SCtrB+(DDp-Dd)*SCtrA)/(Dx-Dr);
    end; {if FIRST<>1 then}
  END; {if Ky=1 then}

end; {with DataMod do}
end; {procedure Transfer}
{-----------------------}


//Calculating the unirrigated area fraction (UU) and the smallest area
//fractions per year (AC,BC,UC), changing the value of Kr if required
procedure AreaFractions;
{----------------------}
var I : byte;
    TC : real;
BEGIN
  with DataMod do
  BEGIN
//Determining permanent area fractions AC, BC, UC
    FOR I:=1 TO NS DO
    begin
      UU[I]:=1-AA[i]-BB[I];
      if UU[I]<0.001 then UU[I]:=0;
      if I=1 then
      begin
        AC:=AA[I];
        BC:=BB[I];
        UC:=UU[I];
      end;
      if AA[I]<AC then AC:=AA[I];
      if BB[I]<BC then BC:=BB[I];
      if UU[I]<UC then UC:=UU[I];
    end;


//Setting initial Kr (rotation key = Kr) value in year zero
 	  if XX=0 then ROTA0:=Kr;
    if AnnualCalc and (YearNr>1) then ROTA0:=ROTAI;

//Checking the value of Kr
   TC:=AC+BC+UC;
	 Kr:=4;
  if (ROTA0=0) or (ROTA0=10) then
  begin
    if TC>0.999 then Kr:=0;
    if TC<1 then
    begin
      if (AC>0) and (BC>0) and (UC>0) then Kr:=10;
      if (AC>0) and (BC>0) and (UC<0.001) then Kr:=5;
      if (AC>0) and (UC>0) and (BC<0.001) then Kr:=6;
      if (BC>0) and (UC>0) and (AC<0.001) then Kr:=7;
      if (AC>0) and (BC<0.001) and (UC<0.001) then Kr:=2;
      if (BC>0) and (AC<0.001) and (UC<0.001) then Kr:=3;
      if (UC>0) and (AC<0.001) and (BC<0.001) then Kr:=1;
    end;
  end;
  if ROTA0=5 then
  begin
    if (AC>0) and (BC>0) then Kr:=5;
    if (AC>0) and (BC<0.001) then Kr:=2;
  end;
  if ROTA0=6 then
  begin
    if (AC>0) and (UC>0) then Kr:=6;
    if (AC>0) and (UC<0.001) then Kr:=2;
  end;
  if ROTA0=7 then
  begin
    if (BC>0) and (UC>0) then Kr:=7;
    if (BC>0) and (UC<0.001) then Kr:=3;
  end;
  if ROTA0=2 then
     if (AC>0) then Kr:=2;
  if ROTA0=3 then
     if (BC>0) then Kr:=3;
  if ROTA0=1 then
     if (UC>0) then Kr:=1;

//FLAG=true when irrigated land is abandoned due to high salinity
   IF FLAG AND (UC>0) THEN
   begin
     if ROTA0<5 then Kr:=1;
     if ROTA0=3 then Kr:=3;
     if (ROTA0=5) or (ROTA0=6) then Kr:=6;
     if ROTA0=7 then Kr:=7;
   end;


//Restoring Kr values when area reductions have been undone
   IF (Kr=1) AND (ROTA0<>1) AND (UC<0.001) THEN
   BEGIN
     FLAG:=false;
     Kr:=ROTA0;
   END;
   IF (Kr=6) AND (ROTA0<>6) AND (UC<0.001) THEN
   BEGIN
     FLAG:=false;
     Kr:=ROTA0;
   END;
   IF (Kr=7) AND (ROTA0<>7) AND (UC<0.001) THEN
   BEGIN
     FLAG:=false;
     Kr:=ROTA0;
   END;

   if AnnualCalc and (YearNr<2) then ROTAI:=ROTA0;

//Warning when Kr was changed
   IF not WarnedRota AND (Kr<>ROTA0) THEN
   BEGIN
     WRITELN (Warnfile);
	   WRITELN (Warnfile, ' The initial rotation key Kr was changed from ',
              ROTA0:2,'  to ',Kr:2);
     WRITELN (Warnfile,'  Later it may or may not change again.');
     WRITELN (Warnfile);
     WarnedRota:=true;
   END;

  END; {with DataMod do}
END; {procedure AreaFractions;}
{-----------------------------}


procedure InitSal;
{----------------}
//Setting initial values of soil salinities depending on Kr
begin with DataMod do
begin

  SCRA:=CA0;
  SCRU:=CU0;
  SCRB:=CB0;
  if Kr=1 then
     if UU[1]<1 then SCRA:=(AA[1]*CA0+BB[1]*CB0)/(1-UU[1])
        else SCRA:=0;
  if Kr=2 then
     if AA[1]<1 then SCRB:=(BB[1]*CB0+UU[1]*CU0)/(1-AA[1])
        else SCRB:=0;
  if Kr=3 then
     if BB[1]<1 then SCRA:=(AA[1]*CA0+UU[1]*CU0)/(1-BB[1])
        else SCRA:=0;
  if Kr=4 then
  begin
    SCRA:=CA0;
    SCRB:=CA0;
    SCRU:=CA0;
  end;
  if Kr>4 then
  begin
    if AA[1]+BB[1]+UU[1]>0 then
       SCRZ:=(AA[1]*CA0+BB[1]*CB0+UU[1]*CU0)/(AA[1]+BB[1]+UU[1])
    else SCRZ:=0;
  end;

  if (Kr=1) and (CU0<=0) then SCRU:=CA0;
  if (Kr=2) and (CA0<=0) then SCRA:=CB0;
  if (Kr=3) and (CB0<=0) then SCRB:=CA0;
  if (Kr=5) and (CU0<=0) then
      if AA[1]+BB[1]>0 then SCRU:=(AA[1]*CA0+BB[1]*CB0)/(AA[1]+BB[1]);
  if (Kr=6) and (CB0<=0) then
      if AA[1]+UU[1]>0 then SCRB:=(AA[1]*CA0+UU[1]*CU0)/(AA[1]+UU[1]);
  if (Kr=7) and (CA0<=0) then
      if BB[1]+UU[1]>0 then SCRA:=(BB[1]*CB0+UU[1]*CU0)/(BB[1]+UU[1]);

//Salinity adjustments when areas change under annual calculations
  if (Ky=1) and (YI>1) then
  begin
    DA:=AC-ACP;
    DB:=BC-BCP;
    DU:=UC-UCP;
    if Kr=1 then
    begin
      if (DU>0) then SCRU:=(UCP*CU0+DU*CA0)/(UCP+DU);
      if (DA>0) then SCRA:=(ACP*CA0+DA*CU0)/(ACP+DA);
    end;
    if Kr=2 then
    begin
      if (DA>0) then SCRA:=(ACP*CA0+DA*CB0)/(ACP+DA);
      if (DB>0) then SCRB:=(BCP*CB0+DB*CA0)/(BCP+DB);
    end;
    if Kr=3 then
    begin
      if (DB>0) then SCRB:=(BCP*CB0+DB*CA0)/(BCP+DB);
      if (DA>0) then SCRA:=(ACP*CA0+DA*CB0)/(ACP+DA);
    end;
    if Kr=5 then
    begin
      if (DB>0) then SCRB:=(BCP*CB0+DB*CA0+DB*CU0)/(BCP+DB+DB);
      if (DA>0) then SCRA:=(ACP*CA0+DA*CB0+DA*CU0)/(ACP+DA+DA);
    end;
    if Kr=6 then
    begin
      if (DA>0) then SCRA:=(ACP*CA0+DA*CB0+DA*CU0)/(ACP+DA+DA);
      if (DU>0) then SCRU:=(UCP*CU0+DU*CA0+DU*CB0)/(UCP+DU+DU);
    end;
    if Kr=6 then
    begin
      if (DB>0) then SCRB:=(BCP*CB0+DB*CA0+DB*CU0)/(ACP+DB+DB);
      if (DU>0) then SCRU:=(UCP*CU0+DU*CA0+DU*CB0)/(UCP+DU+DU);
    end;
  end; {if (Ky=1) and (YI>1) then}

//Intial values of SCRZ
    if Kr>4 then
    begin
      if AA[1]+BB[1]+UU[1]>0 then
         SCrZ:=(AA[1]*SCRA+BB[1]*SCRB+UU[1]*SCRU)/(AA[1]+BB[1]+UU[1])
      else SCRZ:=0;
    end;

//Initial settings of salinity
  IF XX=0 THEN with DataMod do
  BEGIN
    IF (Kr=0) AND (AC>=BC) AND (UU[1]<0.001) THEN SCRU:=SCRA;
    IF (Kr=0) AND (AC<BC) AND (UU[1]<0.001) THEN SCRU:=SCRB;
    IF (Kr=1) AND (AC>=BC) AND (UU[1]<0.001) THEN SCRU:=SCRA;
    IF (Kr=1) AND (AC<BC) AND (UU[1]<0.001) THEN SCRU:=SCRB;
	 END;

end; {with DataMod do}
end; {procedure InitSal}
{----------------------}


procedure WriteOutputData (var AA,BB,BYP,DS,DRR,IRA,IRB,PLC,UU,FF,WEL : real);
{---------------------------------------------------------------------------}
var YR : integer;
    DSR, AIR, BIR : real;
    TmpFile : string;
begin with DataMod do
begin

//Writing year and season in the output file
  YR:=YI+(XX+NS-1) div NS;
  DSR:=DS;
  IF XX=0 THEN DSR:=0;
  WRITELN (OUTFILE);
  TmpFile:=changefileext (FileName,'.INP');
  Writeln (OutFile,'  YEAR: ',YR:4,'   Name of input (data) file used:',
                   TmpFile);
  Writeln (OutFile,'  **********');
  Writeln (OutFile,'  Season:',YY:3,'   Duration:',DSR:5:1, '  months.');
  WRITELN (OUTFILE);
  IF (Ky=1) AND (YY=NS) THEN YI:=YI+1;
  IF (XX=0) THEN YI:=0;

//Canal/field irrigation
  AIR:=IRA;
  BIR:=IRB;
  IF AA<0.001 THEN AIR:=0;
  IF BB<0.001 THEN BIR:=0;
  IF (XX=0) THEN  CIT:=PLC+IRA*AA+IRB*BB+DRR+FF*Wel;
  IF (IRA+IRB<0.001) THEN CIT:=0;
  IF (IRA+IRB<0.001) THEN CIR:=0;
	IF (AA>0.001) AND (BB>0.001) THEN
      Writeln (OutFile,'  It  = ',format('%10.3e',[CIT]),'    Is  = ',
               format('%10.3e',[CIR]),'    IaA = ',format('%10.3e',[AIR]),
               '    IaB = ',format('%10.3e',[BIR]));
	IF (AA>0.001) AND (BB<0.001) THEN
      Writeln (OutFile,'  It  = ',format('%10.3e',[CIT]),'    Is  = ',
               format('%10.3e',[CIR]),'    IaA = ',format('%10.3e',[AIR]),
               '    IaB = ','     -    ');
	IF (AA<0.001) AND (BB>0.001) THEN
      Writeln (OutFile,'  It  = ',format('%10.3e',[CIT]),'    Is  = ',
               format('%10.3e',[CIR]),'    IaA = ','     -    ',
               '    IaB = ',format('%10.3e',[BIR]));
	IF (AA<0.001) AND (BB<0.001) THEN
      Writeln (OutFile,'  It  = ','     -    ','    Is  = ',
               '     -    ','    IaA = ','     -    ',
               '    IaB = ','     -    ');

//Efficiencies, sufficiencies, EaU (evaporation unirrigated U-land), bypass
	IF (AA>0.001) AND (BB>0.001) THEN
  BEGIN
      Writeln (OutFile,'  FfA = ',format('%10.3e',[EFA]),'    FfB = ',
               format('%10.3e',[EFB]),'    FfT = ',format('%10.3e',[EFT]),
               '    Io  = ',format('%10.3e',[BYP]));
      IF UU>0.001 THEN
         Writeln (OutFile,'  JsA = ',format('%10.3e',[SUA]),'    JsB = ',
                  format('%10.3e',[SUB]),'    EaU = ',format('%10.3e',[EPUC]))
      ELSE
         Writeln (OutFile,'  JsA = ',format('%10.3e',[SUA]),'    JsB = ',
                  format('%10.3e',[SUB]),'    EaU = ','     -    ');
  END;
	IF (AA>0.001) AND (BB<0.001) THEN
  BEGIN
      Writeln (OutFile,'  FfA = ',format('%10.3e',[EFA]),'    FfB = ',
               '     -    ','    FfT = ',format('%10.3e',[EFT]),
               '    Io  = ',format('%10.3e',[BYP]));
      IF UU>0.001 THEN
         Writeln (OutFile,'  JsA = ',format('%10.3e',[SUA]),'    JsB = ',
                  '     -    ','    EaU = ',format('%10.3e',[EPUC]))
      ELSE
         Writeln (OutFile,'  JsA = ',format('%10.3e',[SUA]),'    JsB = ',
                  '     -    ','    EaU = ','     -    ');
  END;
	IF (AA<0.001) AND (BB>0.001) THEN
  BEGIN
      Writeln (OutFile,'  FfA = ','     -    ','    FfB = ',
               format('%10.3e',[EFB]),'    FfT = ',format('%10.3e',[EFT]),
               '    Io  = ',format('%10.3e',[BYP]));
      IF UU>0.001 THEN
         Writeln (OutFile,'  JsA = ','     -    ','    JsB = ',
                  format('%10.3e',[SUB]),'    EaU = ',format('%10.3e',[EPUC]))
      ELSE
         Writeln (OutFile,'  JsA = ','     -    ','    JsB = ',
                  format('%10.3e',[SUB]),'    EaU = ','     -    ');
  END;
	IF (AA<0.001) AND (BB<0.001) THEN
  BEGIN
      Writeln (OutFile,'  FfA = ','     -    ','    FfB = ',
               '     -    ','    FfT = ','     -    ',
               '    Io  = ',format('%10.3e',[BYP]));
      Writeln (OutFile,'  JsA = ','     -    ','    JsB = ',
               '     -    ','    EaU = ',format('%10.3e',[EPUC]));
  END;

//Determining net values of percolation and capillary rise
  if (AA>0.001) then
  begin
    if (caa>=pla) then
    begin
      caa:=caa-pla;
      pla:=0;
    end else
    begin
      pla:=pla-caa;
      caa:=0;
    end;
  end else
  begin
	  caa:=0;
	  pla:=0;
	end;
  if (BB>0.001) then
  begin
    if (cab>=plb) then
    begin
      cab:=cab-plb;
      plb:=0;
    end else
    begin
      plb:=plb-cab;
      cab:=0;
    end;
  end else
  begin
	  cab:=0;
	  plb:=0;
	end;
  if (UU>0.001) then
  begin
    if (cau>=plu) then
    begin
      cau:=cau-plu;
      plu:=0;
    end else
    begin
      plu:=plu-cau;
      cau:=0;
    end;
  end else
  begin
	  cau:=0;
	  plu:=0;
	end;

//Writing percolation and capillary rise
  PLT:=AA*PLA+BB*PLB+UU*PLU;
  CAT:=AA*CAA+BB*CAB+UU*CAU;
  IF  (UU>0.001) THEN
  BEGIN
  	IF (AA>0.001) AND (BB>0.001) THEN
    BEGIN
      Writeln (OutFile,'  LrA = ',format('%10.3e',[PLA]),'    LrB = ',
               format('%10.3e',[PLB]),'    LrU = ',format('%10.3e',[PLU]),
               '    LrT = ',format('%10.3e',[PLT]));
      Writeln (OutFile,'  RrA = ',format('%10.3e',[CAA]),'    RrA = ',
               format('%10.3e',[CAB]),'    RrU = ',format('%10.3e',[CAU]),
               '    RrT = ',format('%10.3e',[CAT]));
    END;
  	IF (AA>0.001) AND (BB<0.001) THEN
    BEGIN
      Writeln (OutFile,'  LrA = ',format('%10.3e',[PLA]),'    LrB = ',
               '     -    ','    LrU = ',format('%10.3e',[PLU]),
               '    LrT = ',format('%10.3e',[PLT]));
      Writeln (OutFile,'  RrA = ',format('%10.3e',[CAA]),'    RrB = ',
               '     -    ','    RrU = ',format('%10.3e',[CAU]),
               '    RrT = ',format('%10.3e',[CAT]));
    END;
  	IF (AA<0.001) AND (BB>0.001) THEN
    BEGIN
        Writeln (OutFile,'  LrA = ','     -    ','    LRB = ',
                 format('%10.3e',[PLB]),'    LrU = ',format('%10.3e',[PLU]),
                 '    RrT = ',format('%10.3e',[PLT]));
        Writeln (OutFile,'  RrA = ','     -    ','    RrB = ',
                 format('%10.3e',[CAB]),'    RrU = ',format('%10.3e',[CAU]),
                 '    RrT = ',format('%10.3e',[CAT]));
    END;
   	IF (AA<0.001) AND (BB<0.001) THEN
    BEGIN
      Writeln (OutFile,'  LrA = ','     -    ','    LRB = ',
               '     -    ','    LrU = ',format('%10.3e',[PLU]),
               '    RrT = ',format('%10.3e',[PLT]));
      Writeln (OutFile,'  RrA = ','     -    ','    RrB = ',
               '     -    ','    RrU = ',format('%10.3e',[CAU]),
               '    RrT = ',format('%10.3e',[CAT]));
    END;
  END;
  IF  (UU<0.001) THEN
  BEGIN
  	IF (AA>0.001) AND (BB>0.001) THEN
    BEGIN
      Writeln (OutFile,'  LrA = ',format('%10.3e',[PLA]),'    LrB = ',
               format('%10.3e',[PLB]),'    LrU = ','     -    ',
               '    LrT = ',format('%10.3e',[PLT]));
      Writeln (OutFile,'  RrA = ',format('%10.3e',[CAA]),'    RrB = ',
               format('%10.3e',[CAB]),'    RrU = ','     -    ',
               '    RrT = ',format('%10.3e',[CAT]));
    END;
  	IF (AA>0.001) AND (BB<0.001) THEN
    BEGIN
      Writeln (OutFile,'  LrA = ',format('%10.3e',[PLA]),'    LrB = ',
               '     -    ','    LrU = ','     -    ',
               '    LrT = ',format('%10.3e',[PLT]));
      Writeln (OutFile,'  RrA = ',format('%10.3e',[CAA]),'    RrB = ',
               '     -    ','    RrU = ','     -    ',
               '    RrT = ',format('%10.3e',[CAT]));
    END;
  	IF (AA<0.001) AND (BB>0.001) THEN
    BEGIN
      Writeln (OutFile,'  LrA = ','     -    ','    LRB = ',
               format('%10.3e',[PLB]),'    LrU = ','     -    ',
               '    RrT = ',format('%10.3e',[PLT]));
      Writeln (OutFile,'  RrA = ','     -    ','    RrB = ',
               format('%10.3e',[CAB]),'    RrU = ','     -    ',
               '    RrT = ',format('%10.3e',[CAT]));
    END;
  END;

//Drain/well discharge, watertable depth
  IF Kd=1 THEN
     Writeln (OutFile,'  Gd  = ',format('%10.3e',[DRC]),'    Ga  = ',
              format('%10.3e',[DRA]),'    Gb  = ',format('%10.3e',[DRB]))
  ELSE
     Writeln (OutFile,'  Gd  = ','     -    ','    Ga  = ',
              '     -    ','    Gb  = ','     -    ');
  IF SALT>0.001 THEN
     Writeln (OutFile,'  Gw  = ',format('%10.3e',[WEL]),'    Dw  = ',
              format('%10.3e',[DWT]),'    Zs  = ',format('%10.3e',[SALT]))
  ELSE
     Writeln (OutFile,'  Gw  = ',format('%10.3e',[WEL]),'    Dw  = ',
              format('%10.3e',[DWT]),'    Zs  = ','     -    ');

//Seasonal and permanent area fractions
  Writeln (OutFile,'  A   = ',format('%10.3e',[AA]),'    B   = ',
           format('%10.3e',[BB]),'    U   = ',format('%10.3e',[UU]));
  IF (Kr=0) OR (Kr=1) OR (Kr=6) or (Kr=7) or (Kr=10) THEN
     Writeln (OutFile,'  Uc  = ',format('%10.3e',[UC]),'    Kr   = ',Kr:3)
  ELSE
     Writeln (OutFile,'  Uc  = ','     -    ','    Kr   = ',Kr:3);

//Soil salinities in rootzone, afhankelijk van Kr
	 IF SCRA<0.001 THEN SCRA:=0;
	 IF SCRB<0.001 THEN SCRB:=0;
	 IF SCRU<0.001 THEN SCRU:=0;
   IF (UC<0.001) THEN UC:=0;
   IF (Kr=0) OR (Kr=10) THEN
   BEGIN
	   IF (AC>0.001) AND (BC>0.001) AND (UC>0.001) THEN
        Writeln (OutFile,'  CrA = ',format('%10.3e',[SCRA]),'    CrB = ',
                 format('%10.3e',[SCRB]),'    CrU = ',format('%10.3e',[SCRU]),
                 '    Cr4 = ','     -    ');
	   IF (AC<0.001) AND (BC>0.001) AND (UC>0.001) THEN
        Writeln (OutFile,'  CrA = ','     -    ','    CrB = ',
                 format('%10.3e',[SCRB]),'    CrU = ',format('%10.3e',[SCRU]),
                 '    Cr4 = ','     -    ');
	   IF (AC>0.001) AND (BC<0.001) AND (UC>0.001) THEN
        Writeln (OutFile,'  CrA = ',format('%10.3e',[SCRA]),'    CrB = ',
                 '     -    ','    CrU = ',format('%10.3e',[SCRU]),
                 '    Cr4 = ','     -    ');
	   IF (AC>0.001) AND (BC>0.001) AND (UC<0.001) THEN
        Writeln (OutFile,'  CrA = ',format('%10.3e',[SCRA]),'    CrB = ',
                 format('%10.3e',[SCRB]),'    CrU = ','     -    ',
                 '    Cr4 = ','     -    ');
	   IF (AC>0.001) AND (BC<0.001) AND (UC<0.001) THEN
        Writeln (OutFile,'  CrA = ',format('%10.3e',[SCRA]),'    CrB = ',
                 '     -    ','    CrU = ','     -    ',
                 '    Cr4 = ','     -    ');
	   IF (AC<0.001) AND (BC>0.001) AND (UC<0.001) THEN
        Writeln (OutFile,'  CrA = ','     -    ','    CrB = ',
                 format('%10.3e',[SCRB]),'    CrU = ','     -    ',
                 '    Cr4 = ','     -    ');
	   IF (AC<0.001) AND (BC<0.001) AND (UC>0.001) THEN
        Writeln (OutFile,'  CrA = ','     -    ','    CrB = ',
                 '     -    ','    CrU = ',format('%10.3e',[SCRU]),
                 '    Cr4 = ','     -    ');
      if AC+BC+UC>0.999 then
        Writeln (OutFile,'  C0* = ','     -    ',
        '    C1* = ','     -    ','    C2* = ','     -    ',
        '    C3* = ','     -    ')
      else
        Writeln (OutFile,'  C0* = ',format('%10.3e',[SCrZ]),
        '    C1* = ','     -    ','    C2* = ','     -    ',
        '    C3* = ','     -    ');

   END; {IF (Kr=0) or (Kr=10) THEN}


   IF Kr=1 THEN
   BEGIN
     Writeln (OutFile,'  CrA = ','     -    ','    CrB = ',
              '     -    ','    CrU = ',format('%10.3e',[SCRU]),
              '    Cr4 = ','     -    ');
     IF (UC<0.999) THEN
        Writeln (OutFile,'  C0* = ','     -    ',
                 '    C1* = ',format('%10.3e',[SCRA]),'    C2* = ',
                 '     -    ','    C3* = ','     -    ')
     ELSE
        Writeln (OutFile,'  C0* = ','     -    ','  C1* = ','     -    ',
                 '    C2* = ',
                 '     -    ','    C3* = ','     -    ');
   END; {IF Kr=1 THEN}

   IF Kr=2 THEN
   BEGIN
     Writeln (OutFile,'  CrA = ',format('%10.3e',[SCRA]),'    CrB = ',
              '     -    ','    CrU = ','     -    ',
              '    Cr4 = ','     -    ');
     IF (AC<0.999) THEN
        Writeln (OutFile,'  C0* = ','     -    ','  C1* = ','     -    ',
                 '    C2* = ',
                 format('%10.3e',[SCRB]),'    C3* = ','     -    ')
     ELSE
        Writeln (OutFile,'  C0* = ','     -    ','  C1* = ','     -    ',
                 '    C2* = ',
                 '     -    ','    C3* = ','     -    ');
   END; {IF Kr=2 THEN}

   IF Kr=3 THEN
   BEGIN
     Writeln (OutFile,'  CrA = ','     -    ','    CrB = ',
              format('%10.3e',[SCRB]),'    CrU = ','     -    ');
     IF (BC<0.999) THEN
        Writeln (OutFile,'  C0* = ','     -    ','  C1* = ','     -    ',
                 '    C2* = ',
                 '     -    ','    C3* = ',format('%10.3e',[SCRA]))
     ELSE
        Writeln (OutFile,'  C0* = ','     -    ','  C1* = ','     -    ',
                 '    C2* = ',
                 '     -    ','    C3* = ','     -    ');
   END; {IF Kr=3 THEN}

   IF Kr=4 THEN
   BEGIN
     Writeln (OutFile,'  CrA = ','     -    ',
              '    CrB = ',
              '     -    ','    CrU = ','     -    ',
              '    Cr4 = ',format('%10.3e',[SCRA]));
     Writeln (OutFile,'  C0* = ','     -    ','  C1* = ','     -    ',
              '    C2* = ',
              '     -    ','    C3* = ','     -    ');
   END; {IF Kr=4 THEN}

  if Kr=5 then
  begin
    writeln (OutFile,'  CrA = ',format('%10.3e',[SCrA]),
    '    CrB = ',format('%10.3e',[SCrB]),'    CrU = ','     -    ',
    '    Cr4 = ','     -    ');
    if AC+BC<0.999 then
       writeln (OutFile,'  C0* = ','     -    ',
       '    C1* = ',format('%10.3e',[SCrZ]),
       '    C2* = ','     -    ','    C3* = ','     -    ')
    else
       writeln (OutFile,'  C0* = ','     -    ',
       '    C1* = ','     -    ','    C2* = ',
       '     -    ','    C3* = ','     -    ');
  end;

  if Kr=6 then
  begin
    writeln (OutFile,'  CrA = ',format('%10.3e',[SCrA]),
    '    CrB = ','     -    ','    CrU = ',format('%10.3e',[SCrU]),
    '    Cr4 = ','     -    ');
    if AC+UC<0.999 then
       writeln (OutFile,'  C0* = ','     -    ',
       '    C1* = ',format('%10.3e',[SCrZ]),
       '    C2* = ','     -    ','    C3* = ','     -    ')
    else
       writeln (OutFile,'  C0* = ','     -    ',
       '    C1* = ','     -    ','    C2* = ',
       '     -    ','    C3* = ','     -    ');
  end;

  if Kr=7 then
  begin
    writeln (OutFile,'  CrA = ','     -    ',
    '    CrB = ',format('%10.3e',[SCrB]),
    '    CrU = ',format('%10.3e',[SCrU]),
    '    Cr4 = ','     -    ');
    if AC+UC<0.999 then
       writeln (OutFile,'  C0* = ','     -    ',
       '    C1* = ',format('%10.3e',[SCrZ]),
       '    C2* = ','     -    ','    C3* = ','     -    ')
    else
       writeln (OutFile,'  C0* = ','     -    ',
       '    C1* = ','     -    ','    C2* = ',
       '     -    ','    C3* = ','     -    ');
  end;

//Other salinities
   IF Kd=1 THEN
      Writeln (OutFile,'  Cxf = ','     -    ','    Cxa = ',
               format('%10.3e',[SCtrA]),'    Cxb = ',format('%10.3e',[SCtrB]),
               '    Cqf = ',format('%10.3e',[SCaq]))
   ELSE
      Writeln (OutFile,'  Cxf = ',format('%10.3e',[SCtr]),'    Cxa = ',
               '     -    ','    Cxb = ','     -    ',
               '    Cqf = ',format('%10.3e',[SCaq]));
   IF (Kd=1) AND (CiNa=0) THEN
   BEGIN
     IF WEL>0.001 THEN
     begin
        if UU<1 then
        Writeln (OutFile,'  Ci  = ',format('%10.3e',[ZIR]),'    Cd  = ',
                 format('%10.3e',[ZDR]),'    Cw  = ',format('%10.3e',[ZWL]))
        else
        Writeln (OutFile,'  Ci  = ','     -    ','    Cd  = ',
                 format('%10.3e',[ZDR]),'    Cw  = ',format('%10.3e',[ZWL]));
     end
     ELSE
     begin
        if UU<1 then
        Writeln (OutFile,'  Ci  = ',format('%10.3e',[ZIR]),'    Cd  = ',
                 format('%10.3e',[ZDR]),'    Cw  = ','     -    ')
        else
        Writeln (OutFile,'  Ci  = ','     -    ','    Cd  = ',
                 format('%10.3e',[ZDR]),'    Cw  = ','     -    ');
     end;
   END;
   IF (Kd=1) AND (CiNa=1) THEN
   BEGIN
     IF WEL>0.001 THEN
     BEGIN
        if UU<1 then
        Writeln (OutFile,'  Ci  = ',format('%10.3e',[ZIR]),'    Cd  = ',
                 format('%10.3e',[ZDR]),'    Cw  = ',format('%10.3e',[ZWL]))
        else
        Writeln (OutFile,'  Ci  = ','     -    ','    Cd  = ',
                 format('%10.3e',[ZDR]),'    Cw  = ',format('%10.3e',[ZWL]));
     END ELSE
        if UU<1 then
        Writeln (OutFile,'  Ci  = ',format('%10.3e',[ZIR]),'    Cd  = ',
                 format('%10.3e',[ZDR]),'    Cw  = ','     -    ')
        else
        Writeln (OutFile,'  Ci  = ','     -    ','    Cd  = ',
                 format('%10.3e',[ZDR]),'    Cw  = ',format('%10.3e',[ZWL]));
   END;
   IF (Kd=0) AND (CiNa=0) THEN
   BEGIN
     IF WEL>0.001 THEN
     BEGIN
      if UU<1 then
      Writeln (OutFile,'  Ci  = ',format('%10.3e',[ZIR]),'    Cd  = ',
               '     -    ','    Cw  = ',format('%10.3e',[ZWL]))
      else
      Writeln (OutFile,'  Ci  = ','     -    ','    Cd  = ',
               '     -    ','    Cw  = ',format('%10.3e',[ZWL]))
     END ELSE
     BEGIN
      if UU<1 then
      Writeln (OutFile,'  Ci  = ',format('%10.3e',[ZIR]),'    Cd  = ',
               '     -    ','    Cw  = ','     -    ')
      else
      Writeln (OutFile,'  Ci  = ','     -    ','    Cd  = ',
               '     -    ','    Cw  = ','     -    ');
     END;
   END;
   IF (Kd=0) AND (CiNa=1) THEN
   BEGIN
     IF WEL>0.001 THEN
     BEGIN
       if UU<1 then
        Writeln (OutFile,'  Ci  = ',format('%10.3e',[ZIR]),'    Cd  = ',
                 '     -    ','    Cw  = ',format('%10.3e',[ZWL]))
       else
        Writeln (OutFile,'  Ci  = ','     -    ','    Cd  = ',
                 '     -    ','    Cw  = ',format('%10.3e',[ZWL]));
     END ELSE
     BEGIN
        if UU<1 then
        Writeln (OutFile,'  Ci  = ',format('%10.3e',[ZIR]),'    Cd  = ',
                 '     -    ','    Cw  = ','     -    ')
        ELSE
        Writeln (OutFile,'  Ci  = ','     -    ','    Cd  = ',
                 '     -    ','    Cw  = ','     -    ');
     END;
   END;
   Writeln (OutFile,' #');

//Calculating and writing frequency distributions of soil salinities
  Frequency (AA,BB,UU,DS,YR);
  WRITELN (OutFile);

//Preparing transfer values if required
  IF (Ky=1) THEN
     IF (YY=NS) AND (XX<>0) THEN
     BEGIN
       IF Kr=1 THEN
       BEGIN
          IF UU<0.999 THEN SCRA:=(SCRA*(1-UC)-SCRU*(UU-UC))/(AA+BB);
          SCRB:=SCRA;
       END;
       IF Kr=2 THEN
       BEGIN
          IF AA<0.999 THEN SCRB:=(SCRB*(1-AC)-SCRA*(AA-AC))/(BB+UU);
          SCRU:=SCRA;
       END;
       IF Kr=3 THEN
       BEGIN
          IF BB<0.999 THEN SCRA:=(SCRA*(1-BC)-SCRB*(BB-BC))/(AA+UU);
          SCRU:=SCRA;
       END;
       IF Kr=4 THEN
       BEGIN
         SCRB:=SCRA;
         SCRU:=SCRA;
       END;
       IF (Kd=1) AND (Dx>0.001) THEN
           SCtr:=((Dd-Dr)*SCtrA+(Dx+Dr-Dd)*SCtrB)/Dx;
       IF Kd=0 THEN
       BEGIN
         SCtrA:=SCtr;
         SCtrB:=SCtr;
         Dd:=Dr;
       END;
//Setting transfer values
    TransValues;
  END;{IF (Ky=1) THEN}

end; {with DataMod do}
end; {WriteOutputData}
{--------------------}


procedure TransValues;                                {part of WriteOutputData}
{--------------------}
begin with DataMod do
begin
  FIRSTp:=FIRST;
  DWTp:=DWT;
  SCRAp:=SCRA;
  SCRBp:=SCRB;
  SCRUp:=SCRU;
  SCtrAp:=SCtrA;
  SCtrBp:=SCtrB;
  SCtrp:=SCtr;
  SCaqp:=SCaq;
  SALTp:=SALT;
  ACp:=AC;
  BCp:=BC;
  UCp:=UC;
  DDp:=Dd;
  KDp:=Kd;
  YIp:=YI;
  KYp:=Ky;
end; {with DataMod do}
end; {procedure TransValues}
{--------------------------}


procedure Frequency (var AA,BB,UU,DS:real; var YR:integer);
{---------------------------------------------------------}
var ALF, BET, A1, A2, A3, A4, B1, B2, B3, B4, U1, U2, U3, U4,
    Z1, Z2, Z3, Z4 : real;
    I : byte;
begin
with DataMod do
  begin
    IF AREA<=100 THEN
    BEGIN
      ALF:=0.25;
      BET:=0.84;
    END ELSE
    IF AREA<=1000 THEN
    BEGIN
      ALF:=0.32;
      BET:=0.81;
    END ELSE
    IF AREA<=1000 THEN
    BEGIN
      ALF:=0.41;
      BET:=0.76;
    END ELSE
    BEGIN
      ALF:=0.52;
      BET:=0.69;
    END;

//Calculating the cumulative frequencies at 20%, 40%, 60%, and 80%
    A1:=SCRA*(BET-ALF*LN(-LN(0.2)));
    A2:=SCRA*(BET-ALF*LN(-LN(0.4)));
    A3:=SCRA*(BET-ALF*LN(-LN(0.5)));
    A4:=SCRA*(BET-ALF*LN(-LN(0.8)));
    B1:=SCRB*(BET-ALF*LN(-LN(0.2)));
    B2:=SCRB*(BET-ALF*LN(-LN(0.4)));
    B3:=SCRB*(BET-ALF*LN(-LN(0.5)));
    B4:=SCRB*(BET-ALF*LN(-LN(0.8)));
    U1:=SCRU*(BET-ALF*LN(-LN(0.2)));
    U2:=SCRU*(BET-ALF*LN(-LN(0.4)));
    U3:=SCRU*(BET-ALF*LN(-LN(0.5)));
    U4:=SCRU*(BET-ALF*LN(-LN(0.8)));
    Z1:=SCRZ*(BET-ALF*LN(-LN(0.2)));
    Z2:=SCRZ*(BET-ALF*LN(-LN(0.4)));
    Z3:=SCRZ*(BET-ALF*LN(-LN(0.5)));
    Z4:=SCRz*(BET-ALF*LN(-LN(0.8)));

//Writing frequency
    IF (Kr=4) THEN
    BEGIN
      WRITELN (OutFile,'Cumulative frequency distribution of Cr4');
      WRITELN (OutFile,'  20%   ',format('%10.3e',[A1]),'  40%   ',
              format('%10.3e',[A2]),'  60%   ',format('%10.3e',[A3]),'  80%   ',
              format('%10.3e',[A4]));
      FOR I:=1 to 6 do
      WRITELN (OutFile);
    END;

    IF Kr=1 THEN
    BEGIN
      IF SCRA>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of C1*');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[A1]),'  40%   ',
              format('%10.3e',[A2]),'  60%   ',format('%10.3e',[A3]),'  80%   ',
              format('%10.3e',[A4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      IF SCRU>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrU');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[U1]),'  40%   ',
              format('%10.3e',[U2]),'  60%   ',format('%10.3e',[U3]),'  80%   ',
              format('%10.3e',[U4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      FOR I:=1 to 3 do
      WRITELN (OutFile);
    END;

    IF (Kr=2) OR (Kr=3) THEN
    BEGIN
      IF SCRA>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of C3*');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[A1]),'  40%   ',
              format('%10.3e',[A2]),'  60%   ',format('%10.3e',[A3]),'  80%   ',
              format('%10.3e',[A4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      IF SCRB>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrB');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[B1]),'  40%   ',
              format('%10.3e',[B2]),'  60%   ',format('%10.3e',[B3]),'  80%   ',
              format('%10.3e',[B4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      FOR I:=1 to 4 do
      WRITELN (OutFile);
    END;

    IF Kr=10 THEN
    BEGIN
      IF (AA>0.001) AND (SCRA>0.001) THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrA');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[A1]),'  40%   ',
              format('%10.3e',[A2]),'  60%   ',format('%10.3e',[A3]),'  80%   ',
              format('%10.3e',[A4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      IF (BB>0.001) AND (SCRB>0.001) THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrB');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[B1]),'  40%   ',
              format('%10.3e',[B2]),'  60%   ',format('%10.3e',[B3]),'  80%   ',
              format('%10.3e',[B4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      IF (UU>0.001) AND (SCRU>0.001) THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrU');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[U1]),'  40%   ',
              format('%10.3e',[U2]),'  60%   ',format('%10.3e',[U3]),'  80%   ',
              format('%10.3e',[U4]));
      end;
      IF (UC>0.001) AND (SCRZ>0.001) THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrU');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[Z1]),'  40%   ',
              format('%10.3e',[Z2]),'  60%   ',format('%10.3e',[Z3]),'  80%   ',
              format('%10.3e',[Z4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
    END;

    IF Kr=0 THEN
    BEGIN
      IF (AA>0.001) AND (SCRA>0.001) THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrA');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[A1]),'  40%   ',
              format('%10.3e',[A2]),'  60%   ',format('%10.3e',[A3]),'  80%   ',
              format('%10.3e',[A4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      IF (BB>0.001) AND (SCRB>0.001) THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrB');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[B1]),'  40%   ',
              format('%10.3e',[B2]),'  60%   ',format('%10.3e',[B3]),'  80%   ',
              format('%10.3e',[B4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      IF (UU>0.001) AND (SCRU>0.001) THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrU');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[U1]),'  40%   ',
              format('%10.3e',[U2]),'  60%   ',format('%10.3e',[U3]),'  80%   ',
              format('%10.3e',[U4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      FOR I:=1 to 2 do
      WRITELN (OutFile);
    END;

    IF Kr=5 THEN
    BEGIN
      IF SCRA>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrA');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[A1]),'  40%   ',
              format('%10.3e',[A2]),'  60%   ',format('%10.3e',[A3]),'  80%   ',
              format('%10.3e',[A4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      IF SCRB>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrB');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[B1]),'  40%   ',
              format('%10.3e',[B2]),'  60%   ',format('%10.3e',[B3]),'  80%   ',
              format('%10.3e',[B4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      IF SCRZ>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of C0*');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[Z1]),'  40%   ',
              format('%10.3e',[Z2]),'  60%   ',format('%10.3e',[Z3]),'  80%   ',
              format('%10.3e',[Z4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      FOR I:=1 to 2 do
      WRITELN (OutFile);
    END;

    IF Kr=6 THEN
    BEGIN
      IF SCRA>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrA');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[A1]),'  40%   ',
              format('%10.3e',[A2]),'  60%   ',format('%10.3e',[A3]),'  80%   ',
              format('%10.3e',[A4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      IF SCRU>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrB');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[U1]),'  40%   ',
              format('%10.3e',[U2]),'  60%   ',format('%10.3e',[U3]),'  80%   ',
              format('%10.3e',[U4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      IF SCRZ>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of C0*');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[Z1]),'  40%   ',
              format('%10.3e',[Z2]),'  60%   ',format('%10.3e',[Z3]),'  80%   ',
              format('%10.3e',[Z4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      FOR I:=1 to 2 do
      WRITELN (OutFile);
    END;

    IF Kr=7 THEN
    BEGIN
      IF SCRB>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrB');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[B1]),'  40%   ',
              format('%10.3e',[B2]),'  60%   ',format('%10.3e',[B3]),'  80%   ',
              format('%10.3e',[B4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      IF SCRU>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of CrB');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[U1]),'  40%   ',
              format('%10.3e',[U2]),'  60%   ',format('%10.3e',[U3]),'  80%   ',
              format('%10.3e',[U4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      IF SCRZ>0.001 THEN
      BEGIN
        WRITELN (OutFile,'Cumulative frequency distribution of C0*');
        WRITELN (OutFile,'  20%   ',format('%10.3e',[Z1]),'  40%   ',
              format('%10.3e',[Z2]),'  60%   ',format('%10.3e',[Z3]),'  80%   ',
              format('%10.3e',[Z4]));
      END ELSE
      BEGIN
        WRITELN (OutFile);
        WRITELN (OutFile);
      END;
      FOR I:=1 to 2 do
      WRITELN (OutFile);
    END;

//Writing frequencies to *.FRQ
   IF (YR=0) AND (YY=1) THEN
   BEGIN
     WRITELN (FrqFile,'  FREQUENCY DISTRIBUTION OF SOIL SALINITY');
     WRITELN (FrqFile,'  "(meant for"',' spreadsheet use)');
     WRITELN (FrqFile);
     IF YY=1 THEN
       WRITELN (FrqFile,' "YEAR:  ',YR:4,'"',' - ',' - ',' "Name of" ',
                ' "input" ',' "file:" ',FileName);
     WRITELN (FrqFile,' "Season: ',YY:3,'"',' - ',' Duration: ',' "',
              format('%5.1f',[DS]),'  months"');
    END;

    IF (YR>0) OR ((YR=0) AND (YY>1)) THEN
    BEGIN
      IF YY=1 THEN WRITELN (FrqFile,' "YEAR:  ',YR:4,'"');
      WRITELN (FrqFile,' "Season: ',YY:3,'"',' - ',' Duration: ',' "',
               format('%5.1f',[DS]),'  months"');
    END;
    WRITELN (FrqFile,'- ','- ','- ','           Cumulative',
             ' "frequency of"',' "soil salinity"');
    WRITELN (FrqFile,'-',' Salinity   ',' -  ',
             '  20%   ','    40%   ','    60%   ','    80%    ');

    IF (Kr=4) THEN
       WRITELN (FrqFile,'- ',' Cr4    ','    -',format('%10.3e',[A1]),
       format('%10.3e',[A2]),format('%10.3e',[A3]),format('%10.3e',[A4]));

    IF (Kr=1) THEN
    BEGIN
      if (SCRA>0.001) then
         WRITELN (FrqFile,'- ',' C1*    ','    -',format('%10.3e',[A1]),
         format('%10.3e',[A2]),format('%10.3e',[A3]),format('%10.3e',[A4]));
      if (SCRU>0.001) then
         WRITELN (FrqFile,'- ',' CrU    ','    -',format('%10.3e',[U1]),
         format('%10.3e',[U2]),format('%10.3e',[U3]),format('%10.3e',[U4]));
    END;

    IF (Kr=2) THEN
    BEGIN
      if (SCRA>0.001) then
         WRITELN (FrqFile,'- ',' C2*    ','    -',format('%10.3e',[A1]),
         format('%10.3e',[A2]),format('%10.3e',[A3]),format('%10.3e',[A4]));
      if (SCRB>0.001) then
         WRITELN (FrqFile,'- ',' CrA    ','    -',format('%10.3e',[B1]),
         format('%10.3e',[B2]),format('%10.3e',[B3]),format('%10.3e',[B4]));
    END;

    IF (Kr=3) THEN
    BEGIN
      if (SCRA>0.001) then
         WRITELN (FrqFile,'- ',' C3*    ','    -',format('%10.3e',[A1]),
         format('%10.3e',[A2]),format('%10.3e',[A3]),format('%10.3e',[A4]));
      if (SCRB>0.001) then
         WRITELN (FrqFile,'- ',' CrB    ','    -',format('%10.3e',[B1]),
         format('%10.3e',[B2]),format('%10.3e',[B3]),format('%10.3e',[B4]));
    END;

    if YY=NS then WRITELN (FrqFile,' #');
    WRITELN (FrqFile);

  end; {with DataMod do}
end; {procedure Frequency}
{------------------------}


procedure Waterbalances (var AAy,BBy,BYPy,DRRy,DSy,EPAy,EPBy,EPUy,FFy,
          HIGy,HOGy,IRAy,IRBy,PLCy,RAIy,RCAy,RCBy,SDAy,SDBy,SRIy,SROy,
          STAy,STBy,STUy,UUy,WELy,WTCy : real);
{-------------------------------------------------------------------------}
label 1, 3, 5;
var J1, J2 : byte;
    JJ, YR  : integer;
    CDWR, HCR, NET, PLCC, PX, STO, ZZ : real;
    DOWN, UPWA, DIFA, DIFB, DIFU, Rain : real;
BEGIN;
with DataMod do
BEGIN

//Setting initial values
  J1:=0;
  J2:=0;
  STO:=0;
  DIFC:=0;
  DIFP:=0;
  PLCC:=PLCy;
  Rain:=RAIy;
  YR:=YI+(XX+NS-1) div NS;

//Choosing factor ZZ in dependance of DWT and CDW for determining
//capillary rise and adjustment of surface runoff
1: CDWR:=CDW-0.5*Dr;
   IF CDWR<=0 THEN
     ZZ:=1
   ELSE
   BEGIN
   ZZ:=(CDW-DWT)/CDWR;
   IF ZZ<0 THEN ZZ:=0;
   IF ZZ>1 THEN ZZ:=1;
   END;
//Adjustments for shallow water tabel
  IF DWT<Dr THEN PLCC:=0.5*PLCy;
  IF DWT<0 THEN PLCC:=0;

//Determining actual evapotranspiration and capillary rise through
//AuxWBA with one tertiary subroutine
  AuxWBA (AAy,BBy,EPAy,EPBy,EPUy,IRAy,IRBy,Rain,
          SDAy,SDBy,SRIy,SROy,STAy,STBy,STUy,UUy,ZZ);

//Percolation losses in irrigated A fields
//in cub.m/season per sq.m irrigated A area
  PLA:=Rain+IRAy{-SstA}-EPAC-SDAy;
  IF (AAy<0.001) THEN PLA:=0;

//Setting zero values when fraction A area is very large
  IF (AAy>0.999) THEN
  BEGIN
    PLB:=0;
    PLU:=0;
    GOTO 3;
  END;

//Percolation losses in irrigated B fields
//in cub.m/season per sq.m irrigated B area
  PLB:=Rain+IRBy{-SstB}-EPBC-SDBy;
  IF (BBy<0.001) THEN PLB:=0;

//Setting zero values when fraction B area is very large
  IF (BBy>0.999) THEN
  BEGIN
    PLA:=0;
    PLU:=0;
    GOTO 3;
  END;

//Percolation losses in non irrigated U fields
//in cub.m/season per sq.m unirrigated U area
  PLU:=Rain{-SstU}-EPUC+SRIy-SROy;
  IF (UUy<0.001) THEN PLU:=0;

//Setting zero values when fraction U area is very large
  IF (UUy>0.999) THEN
  BEGIN
    PLA:=0;
    PLB:=0;
  END;

//Total amount of canal irrigation water excluding re-use
//in cub.m/season per sq.m total area
3: CIR:=PLCy+IRAy*AAy+IRBy*BBy;

//Amount of canal water including re-use in cub.m/season pr sq.m total area
  CIT:=CIR+DRRy+FFy*WelY;
//Determining the subsurface drainage

  HCR:=Dd-DWT;
  IF (HCR<=0) OR (Kd=0) THEN
  BEGIN
    DRA:=0;
    DRB:=0;
    DRC:=0;
  END ELSE
  BEGIN
    DRA:=(1-WTCy)*QH2*(HCR*HCR)*DSy*30;
    DRB:=(1-WTCy)*QH1*HCR*DSy*30;
    IF (DWT<=-0.1) THEN DRA:=DRA*2;
    DRC:=DRA+DRB;
  END;

  IF NOT Warn2[YY] AND (DRC<DRRy) THEN
  BEGIN
    WRITELN (Warnfile);
    WRITELN (WarnFile,' The total drainage in year :',YR:3,', season :',
                 YY:2,' is less than the amount of drainage reused');
    WRITELN (WarnFile,' The drainage re-use (DRR) is reduced to equal'+
                      ' the drainage.');
   WRITELN (WarnFile);
   DRRy:=DRC;
   DRR0[YY]:=DRC;
   Warn2[YY]:=true;
  END;
  IF Warn2[YY] AND (DRC<DRRy) THEN
  begin
   DRRy:=DRC;
   DRR0[YY]:=DRC;
  end;
//Total capillary rise (CAT), percolation losses (PLT) and net recharge
//in cub.m/season per sq.m total area
  CAT:=CAA*AAy+CAB*BBy+CAU*UUy;
  PLT:=PLA*AAy+PLB*BBy+PLU*UUy;
//  SstT:=SstA*AAy+SstB*BBy+SstU*UUy;
  NET:=PLCC+PLT{+SstT}+HIGy-HOGy-WELy-CAT-DRC;

//Adjusting the watertable depth DWT stepwise to find equilibrium
//between NET and STO
  IF NET>STO THEN
  BEGIN
    J1:=1;
    JJ:=1;
  END ELSE
  BEGIN
    J2:=1;
    JJ:=-1;
  END;
  IF (J1=1) AND (J2=1) THEN GOTO 5;
  if abs(Net-Sto)<0.001 then goto 5;
  DWT:=DWT-JJ*0.001;

//Finding the applicable effective porespace in relation to depth of watertable
  PX:=1;
  IF (DWT>0) AND (DWT<=Dr) THEN PX:=Per;
  IF (DWT>Dr) AND (DWT<=Dr+Dx) THEN PX:=Pex;
  IF (DWT>Dr+Dx) THEN PX:=Peq;

//Storage due to stepwise change of the watertable
  STO:=STO+JJ*0.001*PX;
  GOTO 1;

//Provisions for waterlogging (DWT<O)
5: if DWT<0 then      {required for salt balances}
   begin
     PLA:=0;
     PLB:=0;
     PLU:=0;
     PLT:=0;
     CAA:=0;
     CAB:=0;
     CAU:=0;
     CAT:=0;
   end;
   if PLA<0 then PLA:=0;
   if PLB<0 then PLB:=0;
   if PLU<0 then PLU:=0;
   if PLT<0 then PLT:=0;
   if CAA<0 then CAA:=0;
   if CAB<0 then CAB:=0;
   if CAU<0 then CAU:=0;
   if CAT<0 then CAT:=0;

   DOWN:=HOGy+WELy+DRC-HIGy;
   UPWA:=0;
   IF DOWN<0 THEN
   BEGIN
     UPWA:=-DOWN;
     DOWN:=0;
   END;

   IF (DWT<0) AND (DWT0<0) THEN
   BEGIN
     DIFA:=0;
     IF (PLA>DOWN) THEN
     BEGIN
       DIFA:=PLA-DOWN;
//       SstA:=DIFA;
       PLA:=DOWN;
     END;
     DIFB:=0;
     IF (PLB>DOWN) THEN
     BEGIN
       DIFB:=PLB-DOWN;
//       SstB:=DIFB;
       PLB:=DOWN;
     END;
     DIFU:=0;
     IF (PLU>DOWN) THEN
     BEGIN
       DIFU:=PLU-DOWN;
//       SstU:=DIFU;
       PLU:=DOWN;
     END;
     DIFP:=DIFP+AAy*DIFA+BBy*DIFB+UUy*DIFU;
   END;
   IF (DWT>=0) AND (DWT0<0) THEN
   BEGIN
     PLA:=PLA+DIFP;
     PLB:=PLB+DIFP;
     PLU:=PLU+DIFP;
   END;

   IF (DWT<0) AND (DWT0<0) THEN
   BEGIN
     DIFA:=0;
     IF (CAA>UPWA) THEN
     BEGIN
       DIFA:=CAA-UPWA;
       CAA:=UPWA;
     END;
     DIFB:=0;
     IF (CAB>UPWA) THEN
     BEGIN
       DIFB:=CAB-UPWA;
       CAB:=UPWA;
     END;
     DIFU:=0;
     IF (CAU>UPWA) THEN
     BEGIN
       DIFU:=CAU-UPWA;
       CAU:=UPWA;
     END;
     DIFC:=DIFC+AAy*DIFA+BBy*DIFB+UUy*DIFU;
   END;

   IF (DWT>=0) AND (DWT0<0) THEN
   BEGIN
     CAA:=CAA+DIFC;
     CAB:=CAB+DIFC;
     CAU:=CAU+DIFC;
     DIFC:=0
   END;

   IF PLA>CAA THEN
   BEGIN
     PLA:=PLA-CAA;
     CAA:=0;
   END;
   IF PLB>CAB THEN
   BEGIN
     PLB:=PLB-CAB;
     CAB:=0;
   END;
   IF PLU>CAU THEN
   BEGIN
     PLU:=PLU-CAU;
     CAU:=0;
   END;
   IF CAA>PLA THEN
   BEGIN
     CAA:=CAA-PLA;
     PLA:=0;
   END;
   IF CAB>PLB THEN
   BEGIN
     CAB:=CAB-PLB;
     PLB:=0;
   END;
   IF CAU>PLU THEN
   BEGIN
     CAU:=CAU-PLU;
     PLU:=0;
   END;

   IF AAy<0.001 THEN PLA:=0;
   IF BBy<0.001 THEN PLB:=0;
   IF UUy<0.001 THEN PLU:=0;
   IF AAy<0.001 THEN CAA:=0;
   IF BBy<0.001 THEN CAB:=0;
   IF UUy<0.001 THEN CAU:=0;

//Redefining total capillary rise (CAT) and percolation (PLT)
   CAT:=CAA*AAy+CAB*BBy+CAU*UUy;
   PLT:=PLA*AAy+PLB*BBy+PLU*UUy;

END; {with DataMod do}
END; {procedure Waterbalances}
{----------------------------}


//Determining actual evapotranspiration and capillary rise through
//AuxWBA with one tertiary subroutine: AuxWBA1  (under Waterbalances)
procedure AuxWBA (var AA1,BB1,EPA1,EPB1,EPU1,IRA1,IRB1,RAI1,
          SDA1,SDB1,SRI1,SRO1,STA1,STB1,STU1,UU1,ZZ1 : real);
{-----------------------------------------------------------}
begin with DataMod do
begin

//Determining actual evapotransp. and cap. rise in unirr. land
  IF UU1<0.001 THEN
  BEGIN
    EPUC:=0;
    PLU:=0;
    CAU:=0;
    SRO1:=0;
  END ELSE
      AuxWBA1 (CAU,EPU1,EPUC,SRI1,RAI1,SRO1,STU1,ZZ1);

//Determining act. evapotr. and cap. rise in irrigated A land
  IF AA1<0.001 THEN
  BEGIN
    EPAC:=0;
    PLA:=0;
    CAA:=0;
    SDA1:=0;
  END ELSE
      AuxWBA1 (CAA,EPA1,EPAC,IRA1,RAI1,SDA1,STA1,ZZ1);

//Determining act. evapotr. and cap. rise in irrigated B land
  IF BB1<0.001 THEN
  BEGIN
    EPBC:=0;
    PLB:=0;
    CAB:=0;
    SDB1:=0;
  END ELSE
      AuxWBA1 (CAB,EPB1,EPBC,IRB1,RAI1,SDB1,STB1,ZZ1);

end; {with DataMod do}
end; {procedure AuxWBA}
{---------------------}


//Determining act. evapotr. and cap. rise
procedure AuxWBA1 (var CA,EP0,EP1,IR,RAIN,SD,ST,ZZ2 : real);
{----------------------------------------------------------}
var ES:real;
begin

  ES:=RAIN+IR-SD;
  IF ST>0 THEN
  BEGIN
    IF (ES>=EP0/ST) OR (DWT<0) THEN
    BEGIN
      EP1:=EP0;
      CA:=0;
    END ELSE
    BEGIN
      EP1:=ST*ES+ZZ2*(EP0-ST*ES);
      CA:=EP1-ES;
      IF CA<0 THEN CA:=0;
    END;
  END;

end; {procedure AuxWBA1}
{----------------------}


procedure SaltBalances (var AAy,BBy,DRRy,FFy,HIGy,HOGy,IRAy,IRBy,PLCy,RAIy,
                        SDAy,SDBy,SRIy,SROy,UUy,WELy : real);
{-------------------------------------------------------------------------}
var SDLA, SDLB, SDLU, ZIRR, CIRT, IRA, IRB,
    INFA, INFB, INFU, SDAA, SDBB, SROO, RAIN, PLCC, DOWN, UPWA,
    SCX, TSP, SCMA, SCMB, SCMU, TIR, TDR, TSIU, TSCU, TSI, TSO,
    TSIA, TSIB, TCAA, TCAB, TSCA, TSCB, TPLA, TPLB, VF, VDD, VUS,
    SCtrM, SCtrBM, SCtrAM, SCRy, SdrT, Win, Zinf, Zout                  : real;
    ACC, AUB, SRI1, SRI2, SCMZ, Divider, ACCI, ACCO : real;
    SSI, WaterIn, SurfConc, TPL, XareaA, XareaB, XareaU : real;

BEGIN with DataMod do
begin

//Salt concentration of the irrigation (ZIR) drainage (ZDR) and well water (ZWL)
  ZDR:=0;
  ZIRR:=ZIR;
  IF (Kd=1) AND (DRC>0.001) THEN ZDR:=(DRA*SCtrA+DRB*SCtrB)*Flx/DRC;
  ZWL:=Flq*SCaq;
  IF WELy<0.001 THEN ZWL:=0;
  ZIR:=ZIC;
	CIR:=CIT-DRRy-FFy*WELy;
  if CIT>0.001 THEN ZIR:=(CIR*ZIC+FFy*Wely*ZWL+DRRy*ZDR)/(CIT);

//Surface drainage leaching efficiency
  SDLA:=(10+SCRA)/100;
  SDLB:=(10+SCRB)/100;
  SDLU:=(10+SCRU)/100;

//Infiltration,rainfall and surface drainage provision for waterlogging
  if (AAy+BBy)>0 then
  begin
    INFA:=IRAy+AAy*(DRRy+FFy*WelY)/(AAy+BBy);
    INFB:=IRBy+BBy*(DRRy+FFy*WelY)/(AAy+BBy);
  end else
  begin
    INFA:=0;
    INFB:=0;
  end;
  INFU:=SRIy;
  IRA:=IRAy;
  IRB:=IRBy;
  SDAA:=SDAy;
  SDBB:=SDBy;
  SROO:=SROy;
  PLCC:=PLCy;
  RAIN:=RaiY;
  DOWN:=HOGy+WELy+DRC-HIGy;
  UPWA:=0;
  IF DOWN<0 THEN
  BEGIN
    UPWA:=-DOWN;
    DOWN:=0;
  END;
  IF DWT<Dr THEN PLCC:=0.5*PLCy;
  IF DWT<0 THEN
  BEGIN
    INFA:=DOWN;
    INFB:=DOWN;
    INFU:=DOWN;
  END;

//Salt accumulation above the soil surface and adjustment of ZIR
  CirT:=IRA*AAy+IRB*BBy+SRIy*UUy;
  if (SCRA>51) AND (SDAy<0.01) THEN SDAA:=0.01*(SCRA-50);
  if (SCRB>51) AND (SDBy<0.01) THEN SDBB:=0.01*(SCRB-50);
  if (SCRU>51) AND (SROy<0.01) THEN SROO:=0.01*(SCRU-50);
  SdrT:=SDAA*AAy+SDBB*BBy+SROO*UUy;
  Win:=CirT+RAIN-SdrT;
  SCRy:=SCRA;
  if Kr=10 THEN
     SCRy := AAy*SCrA+BBy*SCrB+UC*SCrU+(1-AC-BC-UC)*SCrZ;
  if Kr=0 THEN SCRy := AAy*SCRA+BBy*SCRB+UUy*SCRU;
  if Kr=1 THEN SCRy := (1-UC)*SCRA+Uc*SCRU;
  if Kr=2 THEN SCRy := Ac*SCRA+(1-Ac)*SCRB;
  if Kr=3 THEN SCRy := Bc*SCRB+(1-Bc)*SCRA;
  if Kr=4 THEN SCRy := SCRA;
  if Kr=5 THEN SCRy := AC*SCrA+UC*SCrB+(1-AC-BC)*SCrZ;
  if Kr=6 THEN SCRy := AC*SCrA+UC*SCrU+(1-AC-UC)*SCrZ;
  if Kr=7 THEN SCRy := BC*SCrB+UC*SCrU+(1-BC-UC)*SCrZ;
  Zinf:=ZIR*CirT+Zrain*RAIN+SCRy*UPWA;
  Zout:=ZIR*SDrT+ZIR*DOWN;
  SALT:=SALT+Zinf-Zout;
  SurfConc:=SALT/(Win-DWT);
  IF (DWT>=0) AND (DWT0>=0) THEN SurfConc:=0;
  if SurfConc<0 then SurfConc:=0;
  SSI:=WaterIn*SurfConc;

//Salt concentration of water in the transition zone depending on Kd
  IF Kd=0 THEN SCX:=SCtr
  ELSE SCX:=SCtrA;

//Salt concentrations in the irrigated areas (SCRA=A area, SCRB=B area)
//and in the unirrigated area (SCRU) when Kr=0
  IF Kr=0 THEN
  BEGIN
    AuxSB1 (DOWN,UPWA,INFA,INFB,INFU,Rain,SCMA,SCMB,SCMU,SCX,SDAA,SDBB,
            SDLA,SDLB,SDLU,SROO,SSI {,Well});
//Total salt percolating
    TSP:=Flr*(PLA*SCMA*AAy+PLB*SCMB*BBy+PLU*SCMU*UUy);
    IF DWT<0 THEN TSP:=Flr*(DOWN*SCMA*AAy+DOWN*SCMB*BBy+DOWN*SCMU*UUy);
  END;

//Total irrigation, surface drainage
  TIR:=INFA*AAy+INFB*BBy+INFU*UUy;
  TDR:=SDAA*AAy+SDBB*BBy+SROO*UUy;

//Salt concentration (SCRU) in the permanent fallow area and in the
//rotationally irrigated A and B area (SCRA) when Kr=1
  IF Kr=1 THEN
  BEGIN
    TSIU:=INFU*ZIR-SROy*(SDLU*SCRU)+RAIN*Zrain+SSI;
    IF DWT<0 THEN TSIU:=DOWN*ZIR+UPWA*Flx*SCX;
    IF (DWT>0) AND (DWT0<=0) THEN TSIU:=SSI;
    TSCU:=CAU*Flx*SCX;
    IF DWT<0 THEN TSCU:=UPWA*Flx*SCX;
    IF UC<0.999 THEN
    BEGIN
      TSI:=(TIR-INFU*UC)*ZIR;
      TSO:=(TDR-SROy*UC)*(SDLA*SCRA);
      TSIA:=(TSI-TSO)/(1-UC)+RAIN*Zrain;
      IF DWT<0 THEN TSIA:=DOWN*ZIR+UPWA*Flx*SCX;
      IF (DWT>0) AND (DWT0<=0) THEN TSIA:=SSI;
      TCAA:=(CAT-CAU*UC)/(1-UC);
      TSCA:=TCAA*Flx*SCX;
      IF DWT<0 THEN TSCA:=UPWA*Flx*SCX;
      TPLA:=(PLT-PLU*UC)/(1-UC);
      IF DWT<0 THEN TPLA:=DOWN;
    END ELSE
    BEGIN
      TSIA:=0;
      TCAA:=0;
      TSCA:=0;
      TPLA:=0;
    END;
    AuxSB2 (UC,SCRA,SCMA,SCRA0,SCRU,SCMU,SCRU0,TSCA,TSCU,TSIA,TSIU,TPLA,PLU);
//Total salt percolating
    TSP:=Flr*(TPLA*SCMA*(1-UC)+PLU*SCMU*UC);
    IF DWT<0 THEN TSP:=Flr*(DOWN*SCMA*(1-UC)+DOWN*SCMU*UC);
  END; {IF Kr=1 THEN}

  IF Kr=2 THEN
  BEGIN
    TSIA:=INFA*ZIR-SDAy*(SDLA*SCRA)+RAIN*Zrain;
    IF DWT<0 THEN TSIA:=DOWN*ZIR+UPWA*Flx*SCX;
    IF (DWT>0) AND (DWT0<=0) THEN TSIA:=SSI;
    TSCA:=CAA*Flx*SCX;
    IF DWT<0 THEN TSCA:=UPWA*Flx*SCX;
    IF AC<0.999 THEN
    BEGIN
      TSI:=(TIR-INFA*AC)*ZIR;
      TSO:=(TDR-SDAy*AC)*(SDLA*SCRA);
      TSIB:=(TSI-TSO)/(1-AC)+RAIN*Zrain;
      IF DWT<0 THEN TSIB:=DOWN*ZIR+UPWA*Flx*SCX;
      IF (DWT>0) AND (DWT0<=0) THEN TSIB:=SSI;
      TCAB:=(CAT-CAA*AC)/(1-AC);
      TSCB:=TCAB*Flx*SCX;
      IF DWT<0 THEN TSCB:=UPWA*Flx*SCX;
      TPLB:=(PLT-PLA*AC)/(1-AC);
      IF DWT<0 THEN TPLB:=DOWN;
    END ELSE
    BEGIN
      TSIB:=0;
      TCAB:=0;
      TSCB:=0;
      TPLB:=0;
    END;
    AuxSB2 (AC,SCRB,SCMB,SCRB0,SCRA,SCMA,SCRA0,TSCB,TSCA,TSIB,TSIA,TPLB,PLA);
//Total salt percolating
    TSP:=Flr*(PLA*SCMA*AC+TPLB*SCMB*(1-AC));
    IF DWT<0 THEN TSP:=Flr*DOWN*SCMA*AC+DOWN*SCMB*(1-AC);
  END; {IF Kr=2 THEN}

  IF Kr=3 THEN
  BEGIN
    TSIB:=INFB*ZIR-SDBy*SDLB*SCRB+RAIN*Zrain;
    IF DWT<0 THEN TSIB:=DOWN*ZIR+UPWA*Flx*SCX;
    IF (DWT>0) AND (DWT0<=0) THEN TSIB:=SSI;
    TSCB:=CAB*Flx*SCX;
    IF DWT<0 THEN TSCB:=UPWA*Flx*SCX;
    IF AC<0.999 THEN
    BEGIN
      TSI:=(TIR-INFB*BC)*ZIR;
      TSO:=(TDR-SDBy*BC)*(SDLB*SCRB);
      TSIA:=(TSI-TSO)/(1-BC)+RAIN*Zrain;
      IF DWT<0 THEN TSIA:=DOWN*ZIR+UPWA*Flx*SCX;
      IF (DWT>0) AND (DWT0<=0) THEN TSIA:=SSI;
      TCAA:=(CAT-CAB*BC)/(1-BC);
      TSCA:=TCAA*Flx*SCX;
      IF DWT<0 THEN TSCA:=UPWA*Flr*SCX;
      TPLA:=(PLT-PLB*BC)/(1-BC);
      IF DWT<0 THEN TPLA:=DOWN;
    END ELSE
    BEGIN
      TSIA:=0;
      TCAA:=0;
      TSCA:=0;
      TPLA:=0;
    END;
    AuxSB2 (BC,SCRA,SCMA,SCRA0,SCRB,SCMB,SCRB0,TSCA,TSCB,TSIA,TSIB,TPLA,PLB);
//Total salt percolating
    TSP:=Flr*(TPLA*SCMA*(1-BC)+PLB*SCMB*BC);
    IF DWT<0 THEN TSP:=Flr*(DOWN*SCMA*(1-BC)+DOWN*SCMB*BC);
  END; {IF Kr=3 THEN}

//Salt concentration in the rootzone (SCRA) when full rotation occurs
  IF Kr=4 THEN
  BEGIN
    TSI:=TIR*ZIR+RAIN*Zrain+CAT*Flx*SCX-TDR*(SDLA*SCRA);
    IF DWT<0 THEN TSI:=DOWN*ZIR+UPWA*Flx*SCX;
    IF (DWT>0) AND (DWT0<=0) THEN TSI:=SSI;
    TPL:=PLT;
    IF DWT<0 THEN TPL:=DOWN;
    AuxSB3 (TPL,SCRA,SCMA,TSI);
//Total salt percolating
    TSP:=Flr*PLT*SCMA;
    IF DWT<0 THEN TSP:=DOWN*SCMA;
  END;

//Salt concentration of the fixed A (SCRA), B (SCRB) and U (SCRU) areas
//and outside (SCRZ) when RotationType=10
  IF Kr=10 THEN
  BEGIN

//Salt accumulation (ACC) and concentration (SCRA) in the rootzone of A area
   ACCI:=INFA*ZIR+CAA*Flx*SCX;
   IF DWT<0 THEN ACCI:=DOWN*ZIR+UPWA*Flx*SCX;
   IF (DWT>0) AND (DWT0<=0) THEN ACCI:=SSI;
   ACCO:=SDAy*SDLA*SCRA0+SCRA0*Flr*PLA;
   IF DWT<0 THEN ACCO:=SDAy*SDLA*SCRA0+SCRA0*Flr*DOWN;
   ACC:=ACCI-ACCO;
   SCrA:=SCRA0+ACC/(Ptr*Dr);
//Finding the average salt concentration of rootzone of A
   IF ACC<0 THEN
   BEGIN
     if SCRA0<=0 then SCRA0:=0.001;
     IF SCrA<=0 THEN SCrA:=0.1*SCRA0;
     SCMA:=sqrt(SCRA0*SCrA);
   END ELSE
     SCMA:=(SCRA0+SCrA)/2;

//Salt accumulation (ACC) and concentration (SCRB) in the rootzone of B area
   ACCI:=INFB*ZIR+CAB*Flx*SCX;
   IF DWT<0 THEN ACCI:=DOWN*ZIR+UPWA*Flx*SCX;
   IF (DWT>0) AND (DWT0<=0) THEN ACCI:=SSI;
   ACCO:=SDBy*SDLB*SCRB0+SCRB0*Flr*PLB;
   IF DWT<0 THEN ACCO:=SDBy*SDLB*SCRB0+SCRB0*Flr*DOWN;
   ACC:=ACCI-ACCO;
   SCrB:=SCRB0+ACC/(Ptr*Dr);
//Finding the average salt concentration of rootzone of B
   IF ACC<0 THEN
   BEGIN
     if SCRB0<=0 then SCRB0:=0.001;
     IF SCrB<=0 THEN SCrB:=0.1*SCRB0;
     SCMB:=sqrt(SCRB0*SCrB);
   END ELSE
     SCMB:=(SCRB0+SCrB)/2;

//Salt accumulation (ACC) and concentration (SCRU) in the rootzone of U area
   ACCI:=INFU*ZIR+CAU*Flx*SCX;
   IF DWT<0 THEN ACCI:=DOWN*ZIR+UPWA*Flx*SCX;
   IF (DWT>0) AND (DWT0<=0) THEN ACCI:=SSI;
   ACCO:=SROy*SDLU*SCRU0+SCRU0*Flr*PLU;
   IF DWT<0 THEN ACCO:=SROy*SDLU*SCRU0+SCRU0*Flr*DOWN;
   ACC:=ACCI-ACCO;
   SCrU:=SCRU0+ACC/(Ptr*Dr);
//Finding the average salt concentration of rootzone of U
   IF ACC<0 THEN
   BEGIN
     if SCRU0<=0 then SCRU0:=0.001;
     IF SCrU<=0 THEN SCrU:=0.1*SCRU0;
     SCMU:=sqrt(SCRU0*SCrU);
   END ELSE
     SCMU:=(SCRU0+SCrU)/2;

//Salt accumulation (ACC) and concentration (SCRZ) in rootzone of outside area
   XareaA:=AAy-AC;
   XareaB:=BBy-BC;
   XareaU:=UUy-UC;
   Divider:=XareaU+XareaA+XareaB;
   ACCI:=(INFU*XareaU+INFA*XareaA+INFB*XareaB)*ZIR/Divider
         +(CAU*XareaU+CAA*XareaA+CAB*XareaB)*Flx*SCX/Divider;
   IF DWT<0 THEN
      ACCI:=(DOWN*XareaU+DOWN*XareaA+DOWN*XareaB)*ZIR/Divider
            +(UPWA*XareaU+UPWA*XareaA+UPWA*XareaB)*Flx*SCX/Divider;
   IF DWT<0 THEN
      ACCI:=ACCI-(CAA*XareaA+CAB*XareaB+CAU*XareaU)*Flr*SCRZ0/Divider;
   IF (DWT>=0) AND (Dwt0<0) THEN ACCI:=SSI;
   ACCO:=(SDAy*XareaA*SDLA+SDBy*XareaB*SDLB+SROy*XareaU*SDLU+PLA*XareaA*Flr
         +PLB*XareaB*Flr+PLU*XareaU*Flr)*ScrZ0/Divider;
   IF DWT<0 THEN
      ACCO:=(SDAy*XareaA*SDLA+SDBy*XareaB*SDLB+SROy*XareaU*SDLU
            +DOWN*Flr*XareaA+DOWN*Flr*XareaB+DOWN*Flr*XareaU)*ScrZ0/Divider;
   ACC:=ACCI-ACCO;
   SCrZ:=SCRZ0+ACC/(Ptr*Dr);
//Finding the average salt concentration of rootzone of outside area
   IF ACC<0 THEN
   BEGIN
     if SCRZ0<=0 then SCRZ0:=0.001;
     IF SCrZ<=0 THEN SCrZ:=0.1*SCRZ0;
     SCMZ:=sqrt(SCRZ0*SCrZ);
   END ELSE
     SCMZ:=(SCRZ0+SCrZ)/2;

//Total salt percolating
    TSP:=Flr*(PLA*SCMA*AAy+PLB*SCMB*BBy+PLU*SCMU*UUy
         +SCMZ*(PLA*XareaA+PLB*XareaB+PLU*XareaU));
    IF DWT<0 then
       TSP:=Flr*(DOWN*SCMA*AAy+DOWN*SCMB*BBy+DOWN*SCMU*UUy
            +SCMZ*(DOWN*XareaA+DOWN*XareaB+DOWN*XareaU));
  END; {IF Kr=10 THEN}

//Salt concentration of the fixed A (SCRA) and B (SCRB) areas
//and outside (SCRZ) when RotationType=5
  IF Kr=5 THEN
  BEGIN
//Salt accumulation (ACC) and concentration (SCRA) in the rootzone of A area
   ACCI:=INFA*ZIR+CAA*Flx*SCX;
   IF DWT<0 THEN ACCI:=DOWN*ZIR+UPWA*Flx*SCX;
   IF (DWT>0) AND (DWT0<=0) THEN ACCI:=SSI;
   ACCO:=SDAy*SDLA*SCRA0+SCRA0*Flr*PLA;
   IF DWT<0 THEN ACCO:=SDAy*SDLA*SCRA0+SCRA0*Flr*DOWN;
   ACC:=ACCI-ACCO;
   SCrA:=SCRA0+ACC/(Ptr*Dr);
   IF (SCrA=0) OR (SCRA0=0) THEN SCMA:=0;
//Finding the average salt concentration of rootzone of A
   IF ACC<0 THEN
   BEGIN
     if SCRA0<=0 then SCRA0:=0.001;
     IF SCrA<=0 THEN SCrA:=0.1*SCRA0;
     SCMA:=sqrt(SCRA0*SCrA);
   END ELSE
     SCMA:=(SCRA0+SCrA)/2;

//Salt accumulation (ACC) and concentration (SCRB) in the rootzone of B area
   ACCI:=INFB*ZIR+CAB*Flx*SCX;
   IF DWT<0 THEN ACCI:=DOWN*ZIR+UPWA*Flx*SCX;
   IF (DWT>0) AND (DWT0<=0) THEN ACCI:=SSI;
   ACCO:=SDBy*SDLB*SCRB0+SCRB0*Flr*PLB;
   IF DWT<0 THEN ACCO:=SDBy*SDLB*SCRB0+SCRB0*Flr*DOWN;
   ACC:=ACCI-ACCO;
   SCrB:=SCRB0+ACC/(Ptr*Dr);
   IF (SCrB=0) OR (SCRB0=0) THEN SCMB:=0;
//Finding the average salt concentration of rootzone of B
   IF ACC<0 THEN
   BEGIN
     if SCRB0<=0 then SCRB0:=0.001;
     IF SCrB<=0 THEN SCrB:=0.1*SCRB0;
     SCMB:=sqrt(SCRB0*SCrB);
   END ELSE
     SCMB:=(SCRB0+SCrB)/2;

//Salt accumulation (ACC) and concentration (SCRZ) in rootzone of outside area
   XareaA:=AAy-AC;
   XareaB:=BBy-BC;
   Divider:=UUy+XareaA+XareaB;
   ACCI:=(INFU*UUy+INFA*XareaA+INFB*XareaB)*ZIR/Divider
         +(CAU*UUy+CAA*XareaA+CAB*XareaB)*Flx*SCX/Divider;
   IF DWT<0 THEN
      ACCI:=(DOWN*UUy+DOWN*XareaA+DOWN*XareaB)*ZIR/Divider
            +(UPWA*UUy+UPWA*XareaA+UPWA*XareaB)*Flx*SCX/Divider;
   IF (DWT>=0) AND (Dwt0<0) THEN ACCI:=SSI;
   ACCO:=(SDAy*XareaA*SDLA+SDBy*XareaB*SDLB+SDLU*SROy*UUy+PLA*Flr*XareaA
         +PLB*Flr*XareaB+PLU*Flr*UUy)*ScrZ0/Divider;
   IF DWT<0 THEN
      ACCO:=(SDAy*XareaA*SDLA+SDBy*XareaB*SDLB+SDLU*Flr*SROy*UUy
            +DOWN*Flr*XareaA+DOWN*Flr*XareaB +DOWN*Flr*UUy)*ScrZ0/Divider;
   ACC:=ACCI-ACCO;
   SCrZ:=SCRZ0+ACC/(Ptr*Dr);
//Finding the average salt concentration of rootzone of outside area
   IF ACC<0 THEN
   BEGIN
     if SCRZ0<=0 then SCRZ0:=0.001;
     IF SCrZ<=0 THEN SCrZ:=0.1*SCRZ0;
     SCMZ:=sqrt(SCRZ0*SCrZ);
   END ELSE
     SCMZ:=(SCRZ0+SCrZ)/2;

//Total salt percolating
    TSP:=Flr*(PLA*SCMA*AAy+PLB*SCMB*BBy+PLU*SCMU*UUy
          +SCMZ*(PLA*XareaA+PLB*XareaB+PLU*UUy));
    IF DWT<0 then
       TSP:=Flr*(DOWN*SCMA*AAy+DOWN*SCMB*BBy+DOWN*SCMU*UUy
             +SCMZ*(DOWN*XareaA+DOWN*XareaB+DOWN*UUy));
  END; {IF Kr=5 THEN}

//Salt concentration of the fixed A (SCRA) and U (SCRU) areas
//and outside (SCRZ) when RotationType=6
  IF Kr=6 THEN
  BEGIN
//Salt accumulation (ACC) and concentration (SCRA) in the rootzone of A area
   ACCI:=INFA*ZIR+CAA*Flx*SCX;
   IF DWT<0 THEN ACCI:=DOWN*ZIR+UPWA*Flx*SCX;
   IF (DWT>0) AND (DWT0<=0) THEN ACCI:=SSI;
   ACCO:=SDAy*SDLA*SCRA0+SCRA0*Flr*PLA;
   IF DWT<0 THEN ACCO:=SDAy*SDLA*SCRA0+SCRA0*Flr*DOWN;
   ACC:=ACCI-ACCO;
   SCrA:=SCRA0+ACC/(Ptr*Dr);
   IF (SCrA=0) OR (SCRA0=0) THEN SCMA:=0;
//Finding the average salt concentration of rootzone of A
   IF ACC<0 THEN
   BEGIN
     if SCRA0<=0 then SCRA0:=0.001;
     IF SCrA<=0 THEN SCrA:=0.1*SCRA0;
     SCMA:=sqrt(SCRA0*SCrA);
   END ELSE
     SCMA:=(SCRA0+SCrA)/2;

//Salt accumulation (ACC) and concentration (SCRB) in the rootzone of B area
   ACCI:=INFU*ZIR+CAU*Flx*SCX;
   IF DWT<0 THEN ACCI:=DOWN*ZIR+UPWA*Flx*SCX;
   IF (DWT>0) AND (DWT0<=0) THEN ACCI:=SSI;
   ACCO:=SROy*SDLU*SCRU0+SCRU0*Flr*PLU;
   IF DWT<0 THEN ACCO:=SROy*SDLU*SCRU0+SCRU0*Flr*DOWN;
   ACC:=ACCI-ACCO;
   SCrU:=SCRU0+ACC/(Ptr*Dr);
   IF (SCrU=0) OR (SCRU0=0) THEN SCMU:=0;
//Finding the average salt concentration of rootzone of B
   IF ACC<0 THEN
   BEGIN
     if SCRU0<=0 then SCRU0:=0.001;
     IF SCrU<=0 THEN SCrU:=0.1*SCRU0;
     SCMU:=sqrt(SCRU0*SCrU);
   END ELSE
     SCMU:=(SCRU0+SCrU)/2;

//Salt accumulation (ACC) and concentration (SCRB) in rootzone of outside area
   XareaA:=AAy-AC;
   XareaU:=UUy-UC;
   Divider:=BBy+XareaA+XareaU;
   ACCI:=(INFU*XareaU+INFA*XareaA+INFB*BBy)*ZIR/Divider
         +(CAU*XareaU+CAA*XareaA+CAB*BBy)*Flx*SCX/Divider;
   IF DWT<0 THEN
      ACCI:=(DOWN*XareaU+DOWN*XareaA+DOWN*BBy)*ZIR/Divider
            +(UPWA*XareaU+UPWA*XareaA+UPWA*BBy)*Flx*SCX/Divider;
   IF (DWT>=0) AND (Dwt0<0) THEN ACCI:=SSI;
   ACCO:=(SDAy*XareaA*SDLA+SDBy*BBy*SDLB+SROy*XareaU*SDLU+PLA*Flr*XareaA
         +PLB*Flr*BBy+PLU*Flr*XareaU)*ScrZ0/Divider;
   IF DWT<0 THEN
      ACCO:=(SDAy*XareaA*SDLA+SDBy*BBy*SDLB+SROy*XareaU*SDLU
            +DOWN*Flr*XareaA+DOWN*Flr*BBy+DOWN*Flr*XareaU)*ScrZ0/Divider;
   ACC:=ACCI-ACCO;
   SCrZ:=SCRZ0+ACC/(Ptr*Dr);
//Finding the average salt concentration of rootzone of outside area
   IF ACC<0 THEN
   BEGIN
     if SCRZ0<=0 then SCRZ0:=0.001;
     IF SCrZ<=0 THEN SCrZ:=0.1*SCRZ0;
     SCMZ:=sqrt(SCRZ0*SCrZ);
   END ELSE
     SCMZ:=(SCRZ0+SCrZ)/2;

//Total salt percolating
    TSP:=Flr*(PLA*SCMA*AAy+PLB*SCMB*BBy+PLU*SCMU*XareaU
          +SCMZ*(PLA*XareaA+PLB*BBy+PLU*XareaU));
    IF DWT<0 then
       TSP:=Flr*(DOWN*SCMA*AAy+DOWN*SCMB*BBy+DOWN*SCMU*XareaU
             +SCMZ*(DOWN*XareaA+DOWN*BBy+DOWN*XareaU));
  END; {IF Kr=6 THEN}

//Salt concentration of the fixed B (SCRB) and U (SCRU) areas
//and outside (SCRZ) when RotationType=6
  IF Kr=7 THEN
  BEGIN
//Salt accumulation (ACC) and concentration (SCRA) in the rootzone of A area
   ACCI:=INFB*ZIR+CAB*Flx*SCX;
   IF DWT<0 THEN ACCI:=DOWN*ZIR+UPWA*Flx*SCX;
   IF (DWT>0) AND (DWT0<=0) THEN ACCI:=SSI;
   ACCO:=SDBy*SDLB*SCRB0+SCRB0*Flr*PLB;
   IF DWT<0 THEN ACCO:=SDBy*SDLB*SCRB0+SCRB0*Flr*DOWN;
   ACC:=ACCI-ACCO;
   SCrB:=SCRA0+ACC/(Ptr*Dr);
   IF (SCrB=0) OR (SCRB0=0) THEN SCMB:=0;
//Finding the average salt concentration of rootzone of A
   IF ACC<0 THEN
   BEGIN
     if SCRB0<=0 then SCRB0:=0.001;
     IF SCrB<=0 THEN SCrB:=0.1*SCRB0;
     SCMB:=sqrt(SCRB0*SCrB);
   END ELSE
     SCMB:=(SCRB0+SCrB)/2;

//Salt accumulation (ACC) and concentration (SCRB) in the rootzone of B area
   ACCI:=INFU*ZIR+CAU*Flx*SCX;
   IF DWT<0 THEN ACCI:=DOWN*ZIR+UPWA*Flx*SCX;
   IF (DWT>0) AND (DWT0<=0) THEN ACCI:=SSI;
   ACCO:=SROy*SDLU*SCRU0+SCRU0*Flr*PLU;
   IF DWT<0 THEN ACCO:=SROy*SDLU*SCRU0+SCRU0*Flr*DOWN;
   ACC:=ACCI-ACCO;
   SCrU:=SCRU0+ACC/(Ptr*Dr);
   IF (SCrU=0) OR (SCRU0=0) THEN SCMU:=0;
//Finding the average salt concentration of rootzone of B
   IF ACC<0 THEN
   BEGIN
     if SCRU0<=0 then SCRU0:=0.001;
     IF SCrU<=0 THEN SCrU:=0.1*SCRU0;
     SCMU:=sqrt(SCRU0*SCrU);
   END ELSE
     SCMU:=(SCRU0+SCrU)/2;

//Salt accumulation (ACC) and concentration (SCRA) in rootzone of outside area
   XareaB:=BBy-BC;
   XareaU:=UUy-UC;
   Divider:=AAy+XareaB+XareaU;
   ACCI:=(INFU*XareaU+INFA*AAy+INFB*XareaB)*ZIR/Divider
         +(CAU*XareaU+CAA*AAy+CAB*XareaB)*Flx*SCX/Divider;
   IF DWT<0 THEN
      ACCI:=(DOWN*XareaU+DOWN*AAy+DOWN*XareaB)*ZIR/Divider
            +(UPWA*XareaU+UPWA*AAy+UPWA*XareaB)*Flx*SCX/Divider;
   IF (DWT>=0) AND (Dwt0<0) THEN ACCI:=SSI;
   ACCO:=(SDAy*AAy*SDLA+SDBy*XareaB*SDLB+SROy*XareaU*SDLU+PLA*Flr*AAy
         +PLB*Flr*XareaB+PLU*Flr*XareaU)*ScrZ0/Divider;
   IF DWT<0 THEN
      ACCO:=(SDAy*AAy*SDLA+SDBy*XareaB*SDLB+SROy*XareaU*SDLU
            +DOWN*Flr*AAy+DOWN*Flr*XareaB+DOWN*Flr*XareaU)*ScrZ0/Divider;
   ACC:=ACCI-ACCO;
   SCrZ:=SCRZ0+ACC/(Ptr*Dr);
//Finding the average salt concentration of rootzone of outside area
   IF ACC<0 THEN
   BEGIN
     if SCRZ0<=0 then SCRZ0:=0.001;
     IF SCrZ<=0 THEN SCrZ:=0.1*SCRZ0;
     SCMZ:=sqrt(SCRZ0*SCrZ);
   END ELSE
     SCMZ:=(SCRZ0+SCrZ)/2;

//Total salt percolating
    TSP:=Flr*(PLA*SCMA*AAy+PLB*SCMB*XareaB+PLU*SCMU*XareaU
          +SCMZ*(PLA*AAy+PLB*XareaB+PLU*XareaU));
    IF DWT<0 then
       TSP:=Flr*(DOWN*SCMA*AAy+DOWN*SCMB*XareaB+DOWN*SCMU*XareaU
             +SCMZ*(DOWN*AAy+DOWN*XareaB+DOWN*XareaU));
  END; {IF Kr=7 THEN}

//Determining the vertical upward seepage from or the vertical downward
//drainage to the aquifer
  VF:=HIGy-HOGy-WELy;
  IF VF<0 THEN
  BEGIN
    VDD:=ABS(VF);
    VUS:=0;
  END ELSE
  BEGIN
    VUS:=VF;
    VDD:=0;
  END;

//Salt concentration in of the water in the transition zone (SCtr) when
//there is no drainage system (Kd=0)
  IF Kd=0 THEN
     AuxSB4A (PLCC,SCtrM,TSP,VDD,VUS)
  ELSE
//Salt concentration in the upper (SCtrA) and lower (SCtrB) transition zone
//when a drainage system is present (Kd=1)
  AuxSB4B (PLCC,{SCaq,SCtrB,}SCtrBM,{SCtrA,} SCtrAM,TSP,VDD,VUS);

//Salt concentration of the drainage water
  ZDR:=0;
  IF (Kd=1) AND (DRC>0.001) THEN
      ZDR:=(DRA*SCtrAM+DRB*SCtrBM)*Flx/DRC;

//Salt concentration of the water in the aquifer (SCaq)
  AuxSB5 (HIGy,HOGy,PLCC,PLT,SCtrBM,SCtrM,VDD,VUS,WELy,Zigw,ZIR);
  IF (DWT>=0) AND (DWT0<0) THEN ZIR:=ZIR+SALT/(TIR+RAIN-TDR-DWT0);
  CiNa:=0;
  IF (DWT>=0) AND (DWT0>=0) THEN
      IF (INFA*AAy=0) AND (INFB*BBy=0) THEN CiNa:=1;
  IF DWT>=0 THEN SALT:=0;
  ZIR:=ZIRR;

end; {with DataMod do}
END; {SaltBalances}
{-----------------}


//AuxSB1 is part of SaltBalances}
//Salt concentrations in the irrigated and unirrigated areas
procedure AuxSB1 (var DOWNx,UPWAx,INFAx,INFBx,INFUx,RAINx,SCMAx,SCMBx,SCMUx,
                  SCXx,SDAx,SDBx,SDLAx,SDLBx,SDLUx,SROx,SSIx{,Wellx} : real);
{-----------------------------------------------------------------------------}
label 1,2,3;
var SCRIA, SCRIB, SCRIU, ACC, ACCI, ACCO : real;

BEGIN with DataMod do
begin

//Setting initial values
  SCRIA:=SCRA0;
  SCRIB:=SCRB0;
  SCRIU:=SCRU0;

//Salt accumulation (ACC) and concentration (SCRA) in the rootzone of A area
1: ACCI:=INFAx*ZIR+RAINx*Zrain+CAA*Flx*SCXx+SSIx{+Wellx*};
   IF DWT<0 THEN ACCI:=DOWNx*ZIR+UPWAx*Flr*SCXx;
   IF (DWT>0) AND (DWT0<=0) THEN ACCI:=SSIx;
   ACCO:=SDAx*(SDLAx*SCRIA)+SCRIA*Flr*PLA;
   IF DWT<0 THEN ACCO:=SDAx*(SDLAx*SCRIA)+SCRIA*Flr*DOWNx;
   ACC:=ACCI-ACCO;
   SCRA:=SCRA0+ACC/(Ptr*Dr);
   IF (SCRA=0) OR (SCRA0=0) THEN
   BEGIN
     SCMAx:=0;
     GOTO 2;
   END;
//Finding the average salt concentration of rootzone of A stepwise
   IF ACC<0 THEN
   BEGIN
     IF SCRA<=0 THEN SCRA:=0.1*SCRA0;
     SCMAx:=sqrt(SCRA0*SCRA);
     SCRIA:=SCRIA-0.01*SCRA0;
     IF SCRIA<=SCMAx THEN GOTO 2;
     GOTO 1;
   END ELSE
     SCMAx:=(SCRA0+SCRA)/2;

//Salt accumulation (ACC) and concentration (SCRB) in the rootzone of B area
2: ACCI:=INFBx*ZIR+RAINx*Zrain+CAB*Flx*SCXx+SSIx;
   IF DWT<0 THEN ACCI:=DOWNx*ZIR+UPWAx*Flr*SCXx;
   IF (DWT>0) AND (DWT0<=0) THEN ACCI:=SSIx;
   ACCO:=SDBx*SDLBx*SCRIB+SCRIB*Flr*PLB;
   IF DWT<0 then ACCO:=SDBx*SDLBx*SCRIB+SCRIB*Flr*DOWNx;
   ACC:=ACCI-ACCO;
   SCRB:=SCRB0+ACC/(Ptr*Dr);
   IF (SCRB=0) OR (SCRB0=0) THEN
   BEGIN
     SCMBx:=0;
     GOTO 3;
   END;
//Finding the average salt concentration of rootzone of B stepwise
   IF ACC<0 THEN
   BEGIN
     IF SCRB<=0 THEN SCRB:=0.1*SCRB0;
     SCMBx:=sqrt(SCRB0*SCRB);
     SCRIB:=SCRIB-0.01*SCRB0;
     IF SCRIB<=SCMBx THEN GOTO 3;
     GOTO 2;
   END ELSE
     SCMBx:=(SCRB0+SCRB)/2;

//Salt accumulation (ACC) and concentration (SCRU) in the rootzone of U area
3: ACCI:=INFUx*ZIR+RAINx*Zrain+CAU*Flx*SCXx;
   IF DWT<0 THEN ACCI:=DOWNx*ZIR+UPWAx*Flr*SCXx;
   IF (DWT>=0) AND (DWT0<0) THEN ACCI:=SSIx;
   ACCO:=SROx*SDLUx*SCRIU+SCRIU*Flr*PLU;
   IF DWT<0 then ACCO:=SROx*SDLUx*SCRIU+SCRIU*Flr*DOWNx;
   ACC:=ACCI-ACCO;
   SCRU:=SCRU0+ACC/(Ptr*Dr);
   IF (SCRU=0) OR (SCRU0=0) THEN
   BEGIN
     SCMUx:=0;
     exit;
   END;
//Finding the average salt concentration of rootzone of B stepwise
   IF ACC<0 THEN
   BEGIN
     IF SCRU<=0 THEN SCRU:=0.1*SCRU0;
     SCMUx:=sqrt(SCRU0*SCRU);
     SCRIU:=SCRIU-0.01*SCRU0;
     IF SCRIU<=SCMUx THEN exit;
     GOTO 3;
   END ELSE
     SCMBx:=(SCRB0+SCRB)/2;

end; {with DataMod do}
END; {procedure AuxSB1}
{---------------------}


//AuxSB2 is part of SaltBalances}
//Salt concentration in the root zone of A, B and/or U areas when
//the rotation key (Kr) is 1, 2 or 3
procedure AuxSB2 (var AUB,SCRp,SCMp,SCOp,SCRq,SCMq,SCOq,TSCp,TSCq,TSIp,TSIq,
                      TPLp,TPLq: real);
//             AuxSB2 (UC,SCRA,SCMA,SCRA0,SCRU,SCMU,SCRU0,TSCA,TSCU,TSIA,TSIU,
//                    TPLA,PLU);
{--------------------------------------------------------------------------}
label 1, 2, 3;
var ACC, SRI1, SRI2 : real;


BEGIN with DataMod do
begin

//Setting initial values
   SRI1:=SCOp;
   SRI2:=SCOq;
   IF AUB>0.999 THEN
   BEGIN
     SCRp:=SCOp;
     GOTO 3
   END;

//Salt accumulation and concentration in the rootzone, mixed area
1: ACC:=TSIp-TPLp*Flr*SRI1+TSCp;
   SCRp:=SCOp+ACC/(Ptr*Dr);
   IF (SCRp=0) OR (SCOp=0) THEN
   BEGIN
     SCMp:=0;
     GOTO 2;
   END;

//Finding the average salt conc. of rootzone stepwise, mixed area
   IF ACC<0 THEN
   BEGIN
     IF SCRp<=0 THEN SCRp:=0.1*SCOp;
     SCMp:=sqrt(SCOp*SCRp);
     SRI1:=SRI1-0.01*SCOp;
     IF SRI1<=SCMp THEN GOTO 2;
     GOTO 1;
   END ELSE
     SCMp:=(SCOp+SCRp)/2;

2: IF AUB<0.001 THEN
   BEGIN
     SCRq:=SCOq;
     exit;
   END;

//Salt accumulation and concentration in the rootzone, fixed area
3: ACC:=TSIq-TPLq*Flr*SRI2+TSCq;
   if (SCOq>16) then ACC:=4*ACC/sqrt(SCOq);
   SCRq:=SCOq+ACC/(Ptr*Dr);
//   IF (SCRq=0) OR (SCOq=0) THEN
//   BEGIN
//     SCMq:=0;
//     exit;
//   END;

//Finding the average salt conc. in the rootzone stepwise, fixed area
   IF ACC<0 THEN
   BEGIN
     IF SCRq<=0 THEN SCRq:=0.1*SCOq;
     SCMq:=sqrt(SCOq*SCRq);
     SRI2:=SRI2-0.01*SCOq;
     IF SRI2<=SCMq THEN exit;
     GOTO 3
   END ELSE
     SCMq:=(SCOq+SCRq)/2;

end; {with DataMod do}

END; {procedure AuxSB2}
{---------------------}


//AuxSB3 is part of SaltBalances}
//Salt concentration in the rootzone (SCRA) when full rotation occurs (Kr=4)
procedure AuxSB3 (var PLF,SCRAx,SCMAx,TSIx : real);
{-------------------------------------------------}
label 1;
var ACCA, SCIA : real;

BEGIN with DataMod do
begin

//Setting initial value
        SCIA:=SCRA0;

//Salt accumulation and concentration in the rootzone
1: ACCA:=TSIx-PLF*Flr*SCIA;
   SCRAx:=SCRA0+ACCA/(Ptr*Dr);

//Finding the average salt concentration of rootzone of total irrigated
//area stepwise
   IF ACCA<0 THEN
   BEGIN
     IF SCRAx<=0 THEN SCRAx:=0.1*SCRA0;
     SCMAx:=sqrt(SCRA0*SCRAx);
     SCIA:=SCIA-0.01*SCRA0;
     IF SCIA<=SCMAx THEN exit;
     GOTO 1;
   END ELSE
     SCMAx:=(SCRA0+SCRAx)/2;

end; {with DataMod do}
END; {procedure AuxSB3}
{---------------------}


//AuxSB4A is part of SaltBalances}
//Salt concentration in of the water in the transition zone (SCtr) when
//there is no drainage system (Kd=0)
procedure AuxSB4A (var PLC,SCtrMx,TSPx,VDDx,VUSx : real);
{------------------------------------------------------}
label 1;
var  ACC,SCI,SCTT : real;

BEGIN with DataMod do
begin

//Setting initial values
  SCI:=SCtr;
  SCTT:=SCtr;

//Salt accumulation and concentration (SCtr) in the transition zone
1: ACC:=TSPx+PLC*ZIR+VUSx*Flq*SCaq-(CAT+VDDx)*Flx*SCTT;
   IF DWT>=Dr+Dx THEN ACC:=TSPx-PLT*Flx*SCTT-CAT*(Flx*SCTT-Flq*SCaq);
   SCtr:=SCI+ACC/(Ptx*Dx);
   IF (SCtr=0) OR (SCI=0) THEN
   BEGIN
     SCtrMx:=0;
     exit;
   END;
//Finding the average salt concentration stepwise
  IF ACC<0 THEN
  BEGIN
    IF SCtr<=0 THEN SCtr:=0.1*SCI;
    SCtrMx:=sqrt(SCI*SCtr);
    SCTT:=SCTT-0.01*SCI;
    IF SCTT<SCtrMx THEN exit;
    GOTO 1;
  END ELSE
    SCtrMx:=(SCI+SCtr)/2;

end; {with DataMod do}
END; {procedure AuxSB4A}
{----------------------}


//AuxSB4B is part of SaltBalances}
//Salt concentration in the upper (SCtrA) and lower (SCtrB) transition zone
//when a drainage system is present (Kd=1)
procedure AuxSB4B (var PLC,{SCaqx,SCtrBx,}SCtrBMx,{SCtrAx,}SCtrAMx,TSPx,VDDx,VUSx : real);
{---------------------------------------------------------------------------}
label 1, 2, 3;
var VUX, VDX, VAX, VUB, SCtrAU, SCI, ACC, PO, SCtrBL, TR : real;

BEGIN with DataMod do
begin

//Auxiliary waterbalance factors
  VUX:=VUSx-DRB;
  IF VUX<0 THEN VUX:=0;
  VDX:=PLT-DRA;
  IF VDX<0 THEN VDX:=0;
  VAX:=CAT+VDX+DRA;
  VUB:=VUX+VDDx+DRB;
//Starting value of the salt concentration in the upper part of
//the transition zone used for iterative calculations
  SCtrAU:=SCtrA;
  SCI:=SCtrAU;
//Salt accumulation and concentration (SCtrA) in the upper part of
//the transition zone
1: ACC:=TSPx+VUX*Flx*SCtrB-VAX*Flx*SCtrAU;
   IF DWT>=Dd THEN
      ACC:=TSPx-PLT*Flx*SCtrAU+CAT*Flx*(SCtrB-SCtrA);
   PO:=Ptx*(Dd-Dr);
   SCtrA:=SCI+ACC/PO;
   IF (SCtrA=0) OR (SCI=0) THEN
   BEGIN
     SCtrAMx:=0;
     GOTO 2;
   END;
//Finding the average salt concentration in the upper part of the
//transition zone stepwise
  IF ACC<0 THEN
  BEGIN
    IF SCtrA<=0 THEN SCtrA:=0.1*SCI;
    SCtrAMx:=sqrt(SCI*SCtrA);
    SCtrAU:=SCtrAU-0.01*SCI;
    IF SCtrAU<SCtrAMx THEN GOTO 2;
    GOTO 1;
  END ELSE
    SCtrAMx:=(SCI+SCtrA)/2;

//Starting value of the salt concentration in the lower part of
//the transition zone used for iterative calculations
2: SCtrBL:=SCtrB;
   SCI:=SCtrBl;
//Salt accumulation and concentration (SCtrB) in lower transition zone
3: ACC:=VDX*Flx*SCtrAMx+PLC*ZIR+VUSx*Flq*SCaq-VUB*Flx*SCtrBL;
   IF DWT>=Dr+Dx THEN
      ACC:=PLT*Flx*(SCtrAMx-SCtrBL)+CAT*(Flq*SCaq-Flx*SCtrBL);
   TR:=Dr+Dx-Dd;
   PO:=Ptx*TR;
   SCtrB:=SCI+ACC/PO;
   IF (SCtrB=0) OR (SCI=0) THEN
   BEGIN
     SCtrBMx:=0;
     exit;
   END;
//Finding the average salt concentration in the lower part of
//the transition zone stepwise
  IF ACC<0 THEN
  BEGIN
    IF SCtrB<=0 THEN SCtrB:=0.1*SCI;
    SCtrBMx:=sqrt(SCI*SCtrB);
    SCtrBL:=SCtrBL-0.01*SCI;
    IF SCtrBL<SCtrBMx THEN exit;
    GOTO 3;
  END ELSE
    SCtrBMx:=(SCI+SCtrB)/2;

end; {with DataMod do}
END; {procedure AuxSB4B}
{----------------------}


//AuxSB5 is part of SaltBalances}
//Salt concentration of the water in the aquifer (SCaq)
procedure AuxSB5 (var HIG,HOG,PLC,PLF,SCtrBMx,SCTMx,VDDx,VUSx,WELx,
                      Zigwx,ZIRx : real);
{---------------------------------------------------------------}
label 1;
var ACC,ACCU,SCtrA,SCI,SCM,SCX : real;

BEGIN with DataMod do
begin

//Determining salinity of the transition zone
  SCX:=SCTMx;
  IF Kd=1 THEN SCX:=SCtrBMx;
//Starting value of the salt concentration in the aquifer used for
//iterative calculations
  SCtrA:=SCaq;
  SCI:=SCaq;
//Salt accumulation (ACC) and concentration (SCaq) in the aquifer
1: ACCU:=HIG*Zigw-(HOG+WELx*Flq)*SCtrA;
  ACC:=ACCU+VDDx*Flx*SCX-VUSx*Flq*SCtrA;
  IF DWT>Dr+Dx THEN ACC:=ACCU+PLF*Flx*SCX+PLC*ZIR-CAT*Flq*SCtrA;
  SCaq:=SCI+ACC/(Ptq*Dq);
//Finding the average salt concentration in the aquifer stepwise
  IF ACC<0 THEN
  BEGIN
    IF SCaq<0 THEN SCaq:=0.1*SCI;
    SCM:=sqrt(SCI*SCaq);
    SCtrA:=SCtrA-0.01*SCI;
    IF SCtrA<SCM THEN exit;
    GOTO 1;
  END;
  
end; {with DataMod do}
END; {procedure AuxSB5}
{---------------------}


//To determine field irrigation efficiencies, canal irrigation efficiencies,
//and sufficiencies of irrigation water
procedure Efficiency (var AAy,BBy,EPAy,EPBy,IRAy,IRBy,RAIy : real);
{-----------------------------------------------------------------}

BEGIN

//Field irrigation efficiency in A area
  EFA:=0;
  IF IRAy>0.001 THEN EFA:=(EPAC-CAA)/(IRAy+RAIy);
  IF EFA>1 THEN EFA:=1;
//Field irrigation efficiency in B area
  EFb:=0;
  IF IRby>0.001 THEN EFb:=(EPbC-CAb)/(IRby+RAIy);
  IF EFB>1 THEN EFB:=1;
//Overall irrigation efficiency
  EFT:=0;
  IF CIT>0.001 THEN
     EFT:=((EPAC-CAA)*AAy+(EPBC-CAB)*BBy)/(CIT+RAIy);
  IF EFT>1 THEN EFT:=1;
//Sufficiency of irrigation water in A area
  SUA:=0;
  IF (EPAC>0.001) AND (EPAy>0.001) THEN SUA:=EPAC/EPAy;
//Sufficiency of irrigation water in B area
  SUB:=0;
  IF (EPBC>0.001) AND (EPBy>0.001) THEN SUB:=EPBC/EPBy;

END; {procedure Efficiency}
{-------------------------}


//Decreasing the seasonal irrigated area fractions when Kf=1 and salinization
//or insufficient irrigation occurs; increasing the irrigation intensity and
//decreasing the bypass if required; reverse when salinity decreases again.                    *!
procedure FarmResponse (var AAy,A0,BBy,B0,BYPy,IRAy,IRBy,STAy,STBy :real);
{------------------------------------------------------------------------}
var AD,AI,BD,BI,FA,FB,BYP0 : real;

BEGIN with DataMod do
begin

//Initial settings
  AI:=AAy;
  BI:=BBy;
  BYP0:=BYPy;

//Assigning salinities when no area reduction has occurred (FLAG=false)
  IF NOT FLAG THEN
  BEGIN
    IF Kr=1 THEN SCRB:=SCRA;
    IF Kr=2 THEN SCRU:=SCRB;
    IF Kr=3 THEN SCRU:=SCRA;
    IF Kr=4 THEN
    BEGIN
      SCRU:=SCRA;
      SCRB:=SCRA;
    END;
  END;

//Reducing the seasonal irrigated area fraction A when salinization
//or insufficiency occurs, adjusting bypass and irrigation accordingly
  IF (AAy>0.001) AND (Kr<>3) THEN
     IF (SCRA>SCRAM) OR (SUA<0.8) THEN
     BEGIN
       FA:=0.95;
       IF SCRA>SCRAM THEN FLAG:=true;
       IF (SCRA>=1.5*SCRAM) OR (SUA<0.7) THEN FA:=0.9;
       AAy:=AAy*FA;
       if AAy<0.01 then AAy:=0;
       IF (EFA<0.8*STAy) AND (SUA>=1) THEN
          BYPy:=BYPy+(1/FA-1)*AAy*IRAy
       ELSE
          IRAy:=IRAy/FA;
     END;

//Reducing the seasonal irrigated area fraction B when salinization
//or insufficiency occurs, adjusting bypass and irrigation accordingly
  IF (BBy>0.001) AND (Kr=3) THEN
     IF (SCRB>SCRBM) OR (SUB<0.8) THEN
     BEGIN
       FB:=0.95;
       IF SCRB>SCRBM THEN FLAG:=true;
       IF (SCRB>=1.5*SCRBM) OR (SUB<0.7) THEN FB:=0.9;
       BBy:=BBy*FB;
       if BBy<0.01 then BBy:=0;
       IF (EFB<0.8*STBy) AND (SUB>=1) THEN
           BYPy:=BYPy+(1/FB-1)*BBy*IRBy
       ELSE
          IRBy:=IRBy/FB;
     END;

     if not Fixed and ((AAy<AI) or (BBy<BI)) then
     begin
        WRITELN (Warnfile);
        WRITELN (WarnFile,' The irrigated area is reduced as a response');
        WRITELN (WarnFile,' of the farmers to soil salinization.)');
        WRITELN (Warnfile);
        Fixed:=true;
     end;
//Increasing the irrigated A area when it was reduced previously
//and the salinity and sufficiency permit; adjusting bypass
  IF (AAy>0.001) AND (Kr<>3) THEN
  BEGIN
    IF (SCRA<SCRAM-0.5) AND (SUA>0.9) THEN
    BEGIN
      FA:=1/0.95;
      AAy:=AAy*FA;
      IF AAy>A0 THEN
      BEGIN
        AAy:=A0;
        FLAG:=false;
      END;
      FA:=AAy/AI;
      BYPy:=BYPy-(1-1/FA)*AAy*IRAy;
      IF ABS(BYPy)<0.001 THEN BYPy:=0;
      IF BYPy<0.001 THEN
      BEGIN
        IRAy:=IRAy/FA;
        BYPy:=BYP0;
      END;
    END;
  END;

//Increasing the irrigated B area when it was reduced previously
//and the salinity and sufficiency permit; adjusting bypass
  IF (BBy>0.001) AND (Kr=3) THEN
  BEGIN
    IF (SCRB<SCRBM-0.5) AND (SUB>0.9) THEN
    BEGIN
      FB:=1/0.95;
      BBy:=BBy*FB;
      IF BBy>A0 THEN
      BEGIN
        BBy:=B0;
        FLAG:=false;
      END;
      FB:=BBy/BI;
      BYPy:=BYPy-(1-1/FB)*BBy*IRBy;
      IF ABS(BYPy)<0.001 THEN BYPy:=0;
      IF BYPy<0 THEN
      BEGIN
        IRBy:=IRBy/FB;
        BYPy:=BYP0;
      END;
    END;
  END;

//Changing soil salinity in the area fractions UC (Kr=1) AC (Kr=2),
//or BC (Kr=3) upon increase of AC or BC
  AD:=AI-AAy;
  BD:=BI-BBy;
  IF (Kr=1) AND (AD<-0.001) THEN SCRA:=(AI*SCRA-AD*SCRU)/(AI-AD);
  IF (Kr=2) AND (BD<-0.001) THEN SCRA:=(BI*SCRB-BD*SCRA)/(BI-BD);
  IF (Kr=3) AND (AD<-0.001) THEN SCRB:=(AI*SCRA-AD*SCRB)/(AI-AD);

end; {with DataMod do}
END; {procedure FarmResponse}
{---------------------------}


//Reducing the irrigation when waterlogging occurs and no reclamation
//takes place (Kf=1)
procedure ReduceIRR (var AAy,BBy,BYPy,IRAy,IRAIy,IRBy,IRBIy,
                         RCAy,RCBy : real);
{----------------------------------------------------------}
var BYP0, FAC, DWTC : real;

BEGIN

//Initial settings
  BYP0:=BYPy;
  FAC:=0;
  IF DWT<0.350 THEN FAC:=0.05;
  IF DWT<0.300 THEN FAC:=0.10;
  IF DWT<0.250 THEN FAC:=0.15;
  IF DWT<0.200 THEN FAC:=0.20;
  IF DWT<0.150 THEN FAC:=0.25;
  IF DWT<=0.100 THEN FAC:=0.30;

  IF RCAy=1 THEN                                              {paddy on A land}
  BEGIN
    FAC:=0;
    IF DWT<-0.100 THEN FAC:=0.05;
    IF DWT<-0.200 THEN FAC:=0.10;
    IF DWT<-0.250 THEN FAC:=0.15;
    IF DWT<-0.300 THEN FAC:=0.20;
    IF DWT<-0.350 THEN FAC:=0.25;
    IF DWT<-0.400 THEN FAC:=0.30;
  END;

//Reducing irrigation IRA at shallow DWT, famers' response, adjusting bypass
  BYPy:=BYPy+FAC*IRAy*AAy;
  IRAy:=IRAy-FAC*IRAy;
//Increasing IRA when the permanently unirrigated area
//has been increased and DWT has therefore increased
  DWTC:=0.6;
  IF RCAy=1 THEN DWTC:=-0.1;
  IF (IRAy<IRAIy) AND (DWT>=DWTC) THEN     //DWT>DWTC does not occur because
  BEGIN                                //ReduceIrr is called only when DWT<0.35
    BYPy:=BYPy-FAC*IRAIy*AAy;
    IRAy:=IRAy+FAC*IRAIy;
    IF ABS(BYPy)<0.001 THEN BYPy:=0;
    IF BYPy<0 THEN
    BEGIN
      IRAy:=IRAIy;
      BYPy:=BYP0;
    END;
  END;

  FAC:=0;
  IF DWT<0.350 THEN FAC:=0.05;
  IF DWT<0.300 THEN FAC:=0.10;
  IF DWT<0.250 THEN FAC:=0.15;
  IF DWT<0.200 THEN FAC:=0.20;
  IF DWT<0.150 THEN FAC:=0.25;
  IF DWT<=0.100 THEN FAC:=0.30;

  IF RCBy=1 THEN                                              {paddy on B land}
  BEGIN
    FAC:=0;
    IF DWT<-0.1001 THEN FAC:=0.05;
    IF DWT<-0.2001 THEN FAC:=0.10;
    IF DWT<-0.2501 THEN FAC:=0.15;
    IF DWT<-0.3001 THEN FAC:=0.20;
    IF DWT<-0.3501 THEN FAC:=0.25;
    IF DWT<-0.4001 THEN FAC:=0.30;
  END;

//Reducing irrigation IRB at shallow DWT, famers' response, adjusting bypass
  BYPy:=BYPy+FAC*IRBy*BBy;
  IRBy:=IRBy-FAC*IRBy;
//Increasing IRB when the permanently unirrigated area
//has been increased and DWT has therefore increased
  DWTC:=0.6;
  IF RCBy=1 THEN DWTC:=-0.1;
  IF (IRBy<IRBIy) AND (DWT>=DWTC) THEN     //DWT>DWTC does not occur because
  BEGIN                                //ReduceIrr is called only when DWT<0.35
    BYPy:=BYPy-FAC*IRBIy*BBy;
    IRBy:=IRBy+FAC*IRBIy;
    IF ABS(BYPy)<0.001 THEN BYPy:=0;
    IF BYPy<0 THEN
    BEGIN
      IRBy:=IRBIy;
      BYPy:=BYP0;
    END;
  END;
  if BYPy<0.001 then BYPy:=0;

END; {procedure ReduceIRR}
{------------------------}


//end of UCalculate
end.

