unit UDataMod;

interface

uses
  SysUtils, Classes, Dialogs, Forms, Controls;

type
  DataSetR = array of real;
  DataSetVar = array [1..7] of DataSetR;
  LineSet  = array of string;

type
  TDataMod = class(TDataModule)
    procedure DataModuleCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    FileName, AnnualName,
    InputFileName,
    FirstName,
    OriginalName,
    Title1, Title2,
    GrTitle1, GrTitle2,
    GrTitle3, Code,
    GroupTitle              : string;
    Variable                : DataSetVar;
    NrOfData : integer;
    FileLine, ErrorLines,
    WarningLines            : LineSet;
    NrOfLines, NrOfWarnings : integer;
    YearNr                  : integer;
    AnnualCalc, FirstTime,
    EndOfAnnual, GroupNotRead : boolean;
    Area                    : real;
    First, Kd, Kf, Kr, Kw,
    Ns, Ny, Ky, NewDat      : byte;
    Ts : array[1..4] of real;
    A, B, RcA, RcB,
    Lc, Io, IaA, Epa,
    IaB, EpB, Pp, EpU,
    FsA, FsB, FsU,
    Gi, Go, SiU, SoU,
    SoA, SoB, Gw, Fw,
    Gu, Frd                  : array[1..4] of real;
    Dr, Ptr, Dx, Ptx, Dq, Ptq : real;
    Per, Flr, Pex, Flx, Peq, Flq : real;
    Cx0, Cq0, Cic, Ch, Cp : real;
    CA0, CB0, CU0, Dw0, Dc : real;
    CAmax, CBmax : real;
    Dd, QH1, QH2 : real;
    Cxa0, Cxb0   : real;
    MainStr      : array [1..7] of string[12];
    NrOfitems    : integer;
    GroupMark    : string[5];
    KwAbsent, Failed : boolean;
    procedure ReadInputFile;
    procedure WriteInputFile(const DocName: string);
    procedure ReadErrorLines
              (var ErrorPresent : boolean; var NrOfErrors : integer);
    procedure ReadWarningLines
              (var WarningPresent : boolean; var NrOfWarnings : integer);
    procedure ReadOutputLines(const DocName: string;
                              var Reading : boolean);
    procedure ReadOutputGroup(Signal : string);
    procedure ReadGroupData;
    procedure WriteOutputGroup(const DocName: string);
    procedure WriteOutputLines(const DocName: string);
    function  StringToFlo(Str : string) : double;
    function  Question(const Query: string): boolean;
  end;

var
  DataMod: TDataMod;

implementation

{$R *.dfm}


{******************************************************************************
 read/write input file
 ******************************************************************************}


procedure TDataMod.ReadInputFile;
{-------------------------------}
var  InFile   : textfile;
     k, MaxNr : byte;
     HlpStr   : string[12];
begin
  Assignfile(InFile, FileName);
  {$I-} Reset(InFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + filename + '"could not be opened. It may be ' +
      'dameged or in use by another program.');
    exit;
  end;
  NrOfLines := 0;
  while not eof (InFile) do
  begin
    readln(InFile);
    NrOfLines := NrOfLines+1;
  end;
  reset (InFile);
  readln(InFile, Title1);
  readln(InFile, Title2);
  readln (InFile);
  KwAbsent:=false;
  Try readln (InFile, Area, Ns, Kd, Kf, Kr, Kw);
      Except on E: exception do
      begin
        Kw:=0;
        KwAbsent:=true;
      end;
  end;
  if not KwAbsent then readln (InFile,HlpStr);
  if Kd=1 then MaxNr:=43 else MaxNr:=39;
  if NrOfLines<MaxNr then
  begin
    readln (InFile, Ny, Ky);
    NewDat := 0;
  end else
  begin
    read (InFile, Ny, Ky, NewDat);
    Try
      readln (InFile,CAmax,CBmax);
      Except on E: exception do
      begin
        Failed:=true;
        CAmax:=8;
        CBmax:=8;
      end;
    end;
  end;
  if not Failed then readln (InFile,HlpStr);
  for k:=1 to Ns do read (InFile,Ts[k]);
  readln (InFile);
  readln (InFile);
  for k:=1 to Ns do read (InFile,A[k],B[k]);
  readln (InFile);
  readln (InFile,HlpStr);
  if HlpStr<>'5.      RcA1' then
  begin
    Showmessage ('This is an old input file. Please check the values'+
                 ' of rice factors RcA and RcB in line 5.');
    for k:=1 to Ns do RcA[k]:=0;
    for k:=1 to Ns do RcB[k]:=0;
  end else
  begin
    for k:=1 to Ns do read (InFile,RcA[k],RcB[k]);
    readln (InFile);
    readln (inFile);
  end;
  for k:=1 to Ns do read (InFile,Lc[k],Io[k]);
  readln (InFile);
  readln (InFile);
  for k:=1 to Ns do read (InFile,IaA[k],EpA[k]);
  readln (InFile);
  readln (InFile);
  for k:=1 to Ns do read (InFile,IaB[k],EpB[k]);
  readln (InFile);
  readln (InFile);
  for k:=1 to Ns do read (InFile,Pp[k],EpU[k]);
  readln (InFile);
  readln (InFile);
  if NewDat<>99 then
  begin
      read (InFile, FsA[1],FsB[1],FsU[1]);
      for k:=2 to 4 do
      begin
        FsA[k] := FsA[1];
        FsB[k] := FsB[1];
        FsU[k] := FsU[1];
      end;
      NewDat := 99;
  end else
  begin
      for k:=1 to Ns do read (InFile,FsA[k],FsB[k]);
      readln (InFile);
      readln (InFile);
      for k:=1 to Ns do read (InFile,FsU[k]);
  end;
  readln (InFile);
  readln (InFile);
  for k:=1 to Ns do read (InFile,Gi[k],Go[k]);
  readln (InFile);
  readln (InFile);
  for k:=1 to Ns do read (InFile,SiU[k],SoU[k]);
  readln (InFile);
  readln (InFile);
  for k:=1 to Ns do read (InFile,SoA[k],SoB[k]);
  readln (InFile);
  readln (InFile);
  for k:=1 to Ns do read (InFile,Gw[k],Fw[k]);
  readln (InFile);
  readln (InFile);
  readln (InFile, Dr, Ptr, Dx, Ptx, Dq, Ptq);
  readln (InFile);
  readln (InFile, Per, Flr, Pex, Flx, Peq, Flq);
  readln (InFile);
  readln (InFile, Cx0, Cq0, Cic, Ch, Cp);
  readln (InFile);
  readln (InFile, CA0, CB0, CU0, Dw0, Dc);
  readln (InFile);
  if Ns=1 then
     readln (InFile, Dd, QH1, QH2, Gu[1])
  else if Ns=2 then
     readln (InFile, Dd, QH1, QH2, Gu[1], Gu[2])
  else if Ns=3 then
     readln (InFile, Dd, QH1, QH2, Gu[1], Gu[2], Gu[3])
  else if Ns=4 then
     readln (InFile, Dd, QH1, QH2, Gu[1], Gu[2], Gu[3], Gu[4])
  else
     Showmessage ('The input file is not correct, select another');
  readln (InFile);
  if Kd=1 then
    if Ns=1 then
       readln (InFile, Cxa0, Cxb0, Frd[1])
    else if Ns=2 then
       readln (InFile, Cxa0, Cxb0, Frd[1], Frd[2])
    else if Ns=3 then
       readln (InFile, Cxa0, Cxb0, Frd[1], Frd[2], Frd[3])
    else if Ns=4 then
       readln (InFile, Cxa0, Cxb0, Frd[1], Frd[2], Frd[3], Frd[4])
    else
       Showmessage ('The input file is not correct, select another');
  Closefile(InFile);
end; {TDataMod.ReadInputFile}
{---------------------------}



procedure TDataMod.WriteInputFile (const DocName: string);
{--------------------------------------------------------}
var  InFile : textfile;
     i,j,k,m : byte;
     HlpStr : string;
     Blines : array [1..23] of String;
     sp0, sp3, sp4, sp5, sp6, sp7, sp8 : string;
begin
  Assignfile(InFile, DocName);
  {$I-} Rewrite(InFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + DocName + '"could not be opened. It ' +
      'may be damaged or in use by another program.');
    exit;
  end;
  writeln(InFile, Title1);
  writeln(InFile, Title2);

  BLines[1]:='Explanatory text:';
  BLines[2]:='Additional explanatory text:';
  BLines[3]:='1.      Area    Ns      Kd      Kf      Kr      Krf';
  BLines[4]:='2.      Ny      Ky      n.a.            CAmax   CBmax';
  BLines[5]:='3.   ';
  BLines[6]:='4.   ';
  BLines[7]:='5.   ';
  BLines[8]:='6.   ';
  BLines[9]:='7.   ';
  BLines[10]:='8.   ';
  BLines[11]:='9.   ';
  BLines[12]:='10.  ';
  BLines[13]:='11.  ';
  BLines[14]:='12.  ';
  BLines[15]:='13.  ';
  BLines[16]:='14.  ';
  BLines[17]:='15.  ';
  BLines[18]:='16.     Dr      Ptr     Dx      Ptx     Dq      Ptq';
  BLines[19]:='17.     Per     Flr     Pex     Flx     Peq     Flq';
  BLines[20]:='18.     Cx0     Cq0     Cic     Ch      Cp';
  BLines[21]:='19.     CA0     CB0     CU0     Dw0     Dc';
  BLines[22]:='20.     Dd      QH1     QH2  ';
  BLines[23]:='21.     Cxa0    Cxb0 ';
  for k:=1 to Ns do
    begin
      str(k:1, HlpStr);
      BLines[5]:= BLines[ 5]+'   Ts'+HlpStr+ '  ';
      BLines[6]:= BLines[ 6]+'   A'+HlpStr+  '      B'+HlpStr+'   ';
      BLines[7]:= BLines[ 7]+'   RcA'+HlpStr+'    RcB'+HlpStr+' ';
      BLines[8]:= BLines[ 8]+'   Lc'+HlpStr+ '     Io'+HlpStr+'  ';
      BLines[9]:= BLines[ 9]+'   IaA'+HlpStr+'    EpA'+HlpStr+' ';
      BLines[10]:=BLines[10]+'   IaB'+HlpStr+'    EpB'+HlpStr+' ';
      BLines[11]:=BLines[11]+'   Pp'+HlpStr+ '     EpU'+HlpStr+' ';
      BLines[12]:=BLines[12]+'   FsA'+HlpStr+'    FsB'+HlpStr+' ';
      BLines[13]:=BLines[13]+'   FsU'+HlpStr+' ';
      BLines[14]:=BLines[14]+'   Gi'+HlpStr+ '     Go'+HlpStr+'  ';
      BLines[15]:=BLines[15]+'   SiU'+HlpStr+'    SoU'+HlpStr+' ';
      BLines[16]:=BLines[16]+'   SoA'+HlpStr+'    SoB'+HlpStr+' ';
      BLines[17]:=BLines[17]+'   Gw'+HlpStr+ '     Fw'+HlpStr+'  ';
      BLines[22]:=BLines[22]+'   Gu'+HlpStr+ '  ';
      BLines[23]:=BLines[23]+'   Frd'+HlpStr+' ';
    end;
  sp3 := '   ';
  sp4 := '    ';
  sp5 := '     ';
  sp6 := '      ';
  sp7 := '       ';
  sp8 := '        ';
  writeln (InFile,BLines[3]);
  writeln (InFile,sp4,Area:8:1,sp4,Ns,sp7,Kd,sp7,Kf,sp7,Kr,sp7,Kw);
  writeln (InFile,BLines[4]);
  writeln (InFile,sp7,' ',Ny,sp6,Ky,sp7,'99',sp4,sp5,CAmax:8:1,CBmax:8:1);
  writeln (InFile,BLines[5]);
  write (InFile,sp3);
  for k:=1 to Ns do write (InFile,Ts[k]:8:1);
  writeln (InFile);
  writeln (InFile,BLines[6]);
  write (InFile,sp4);
  for k:=1 to Ns do write (InFile,A[k]:8:2,B[k]:8:2);
  writeln (InFile);
  writeln (InFile,BLines[7]);
  write (InFile,' ');
  for k:=1 to Ns do write (InFile,RcA[k]:8:0,RcB[k]:8:0);
  writeln (InFile);
  writeln (InFile,BLines[8]);
  write (InFile,sp5);
  for k:=1 to Ns do write (InFile,Lc[k]:8:3,Io[k]:8:3);
  writeln (InFile);
  writeln (InFile,BLines[9]);
  write (InFile,sp5);
  for k:=1 to Ns do write (InFile,IaA[k]:8:3,EpA[k]:8:3);
  writeln (InFile);
  writeln (InFile,BLines[10]);
  write (InFile,sp5);
  for k:=1 to Ns do write (InFile,IaB[k]:8:3,EpB[k]:8:3);
  writeln (InFile);
  writeln (InFile,BLines[11]);
  write (InFile,sp5);
  for k:=1 to Ns do write (InFile,Pp[k]:8:3,EpU[k]:8:3);
  writeln (InFile);
  writeln (InFile,BLines[12]);
  write (InFile,sp4);
  for k:=1 to Ns do write (InFile,FsA[k]:8:2,FsB[k]:8:2);
  writeln (InFile);
  writeln (InFile,BLines[13]);
  write (InFile,sp4);
  for k:=1 to Ns do write (InFile,FsU[k]:8:2);
  writeln (InFile);
  writeln (InFile,BLines[14]);
  write (InFile,sp5);
  for k:=1 to Ns do write (InFile,Gi[k]:8:3,Go[k]:8:3);
  writeln (InFile);
  writeln (InFile,BLines[15]);
  write (InFile,sp5);
  for k:=1 to Ns do write (InFile,SiU[k]:8:3,SoU[k]:8:3);
  writeln (InFile);
  writeln (InFile,BLines[16]);
  write (InFile,sp5);
  for k:=1 to Ns do write (InFile,SoA[k]:8:3,SoB[k]:8:3);
  writeln (InFile);
  writeln (InFile,BLines[17]);
  write (InFile,sp5);
  for k:=1 to Ns do write (InFile,Gw[k]:8:3,Fw[k]:8:3);
  writeln (InFile);
  writeln (InFile,BLines[18]);
  if Dq>9.999 then
    writeln (InFile, sp4, Dr:8:2,Ptr:8:2,Dx:8:2,Ptx:8:2,Dq:9:2,Ptq:7:2)
  else
    writeln (InFile, sp4, Dr:8:2,Ptr:8:2,Dx:8:2,Ptx:8:2,Dq:8:2,Ptq:8:2);
  writeln (InFile,BLines[19]);
  writeln (InFile, sp5, Per:8:3,Flr:7:2,Pex:9:3,Flx:7:2,Peq:9:3,Flq:7:2);
  writeln (InFile,BLines[20]);
  sp0:=sp3;
  if Cx0>=10 then sp0:=sp4;
  j:=8;
  if Cq0>=10 then j:=7;
  i:=8;
  if Ch>=10 then i:=9;
  m:=8;
  if Ch>=10 then m:=7;
  writeln (InFile, sp0, Cx0:8:1, Cq0:8:1, Cic:j:1, Ch:i:1, Cp:m:1);
  writeln (InFile,BLines[21]);
  writeln (InFile, sp3, CA0:8:1, CB0:8:1, CU0:8:1, Dw0:9:2, Dc:8:2);
  if Kd=1 then
  begin
    writeln (InFile,BLines[22]);
    write (InFile, sp4, Dd:8:2, QH1:10:4, QH2:8:4);
    write (InFile,Gu[1]:7:3);
    if Ns>1 then for k:=2 to Ns do write (InFile,Gu[k]:8:3);
    writeln (InFile);
    writeln (InFile,BLines[23]);
    write (InFile, sp3, Cxa0:8:1, Cxb0:8:1,' ');
    for k:=1 to Ns do write (InFile,Frd[k]:8:2);
    writeln (InFile);
  end;
  Closefile(InFile);
end; {TDataMod.WriteInputFile}
{----------------------------}


{******************************************************************************
 read/write output file
 ******************************************************************************}


procedure TDataMod.ReadErrorLines
          (var ErrorPresent : boolean; var NrOfErrors : integer);
{---------------------------------------------------------------}
var ErrorFile : textfile;
    i, k : integer;
    DumLine : string;
begin
    Assignfile (ErrorFile,'Error.lst');
    reset (ErrorFile);
    k:=0;
    while not eof (ErrorFile) do
    begin
      readln (ErrorFile,DumLine);
      k:=k+1;
    end;
    ErrorPresent:=false;
    if k>=3 then
    begin
      reset (ErrorFile);
      ErrorPresent:=true;
      setlength (ErrorLines,k);
      for i:=0 to k-1 do readln (ErrorFile,ErrorLines[i]);
      NrOfErrors:=k;
    end;
    closefile (ErrorFile);
end; {procedure TDataMod.ReadErrorLines}
{--------------------------------------}



procedure TDataMod.ReadWarningLines
          (var WarningPresent : boolean; var NrOfWarnings : integer);
{-------------------------------------------------------------------}
var WarningFile : textfile;
    i, k : integer;
    DumLine : string;
begin
    Assignfile (WarningFile,'warning.lst');
    reset (WarningFile);
    k:=0;
    while not eof (WarningFile) do
    begin
      readln (WarningFile,DumLine);
      k:=k+1;
    end;
    WarningPresent:=false;
    if k>=3 then
    begin
      reset (WarningFile);
      WarningPresent:=true;
      setlength (WarningLines,k);
      for i:=0 to k-1 do readln (WarningFile,WarningLines[i]);
      NrOfWarnings:=k;
    end;
    closefile (WarningFile);
end; {procedure TDataMod.ReadWarningLines}
{----------------------------------------}



procedure TDataMod.ReadOutputLines(const DocName: string;
                                   var Reading : boolean);
{--------------------------------------------------------}
var  OutFile: textfile;
     k: integer;
begin
  Reading:=true;
  AssignFile(OutFile,DocName);
  {$I-} Reset(OutFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + DocName + '"could not be opened. It may be ' +
      'damaged or in use by another program.');
    Reading:=false;
    exit;
  end;
  k:=0;
  while not eof(OutFile) do
  begin
    k:=k+1;
    SetLength (FileLine,k);
    readln(OutFile,FileLine[k-1]);
  end;
  NrOfLines:=k;
  Close(OutFile);
end; {TDataMod.ReadOutputLines}
{-----------------------------}



procedure TDataMod.ReadOutputGroup(Signal : string);
{--------------------------------------------------}
label 1;
var  OutFile  : textfile;
     Code     : string[2];
     DetectIt : string[8];
     HlpStr   : string[8];
     HelpStr  : string[9];
     AuxStr   : string[12];
     k,SkipNr    : integer;
begin
  FileName:=changefileext(FileName,'.res');
  AssignFile(OutFile,FileName);
  {$I-} Reset(OutFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "'+FileName+'"could not be opened. It may be ' +
      'damaged or in use by another program.');
    exit;
  end;
  while not eof (OutFile) do
  begin
    Code:='';
    while Code<>' #' do
          if not eof (OutFile) then readln (OutFile,Code)
          else goto 1;
    for k:=1 to 7 do
        readln (OutFile,DetectIt);
    read (OutFile,DetectIt);
    if DetectIt='' then
    begin
      readln (OutFile,DetectIt);
      read (OutFile,DetectIt);
    end;
    if DetectIt='' then
    begin
      readln (OutFile,DetectIt);
      read (OutFile,DetectIt);
    end;
    if DetectIt='' then
    begin
      readln (OutFile,DetectIt);
      read (OutFile,DetectIt);
    end;
    if DetectIt='' then
    begin
      readln (OutFile,DetectIt);
      read (OutFile,DetectIt);
    end;
    if (DetectIt='  YEAR: ') then
    begin
      readln (OutFile,Ny);
      readln (OutFile);
      readln (OutFile,HelpStr,Ns,AuxStr,Ts[Ns]);
    end;
  end;
1:reset (OutFile);
  NrOfData := 0;
  Ny:=Ny+1;
  SkipNr := 10;
  NrOfitems := 7;
  if (Signal='Cxf') then
  begin
    SkipNr := 12;
    NrOfitems := 7;
  end;
  if (Signal='Gd') then
  begin
    SkipNr := 6;
    NrOfitems := 6;
  end;
  if (Signal='LrA') then
  begin
    SkipNr := 4;
    NrOfitems := 4;
  end;
  if (Signal='RrA') then
  begin
    SkipNr := 5;
    NrOfitems := 4;
  end;
  if (Signal='It') then
  begin
    SkipNr := 1;
    NrOfitems := 5;
  end;
  if (Signal='FfA') then
  begin
    SkipNr := 2;
    NrOfitems := 6;
  end;
  if (Signal='A') then
  begin
    SkipNr := 8;
    NrOfitems := 5;
  end;
  DetectIt:='';
  while NrOfData<Ny*Ns do
  begin
    while not (DetectIt='  Season') do
          readln(OutFile,DetectIt);
    NrOfData := NrOfData+1;
    DetectIt := '';

   for k:=1 to SkipNr do
       readln (OutFile,HlpStr);
   if (Signal='CrA') or (Signal='Cxf') then
    begin
      readln (OutFile,HlpStr,MainStr[1],HlpStr,MainStr[2],
                      HlpStr,MainStr[3],HlpStr,MainStr[4]);
      readln (OutFile,HlpStr,MainStr[5],HlpStr,MainStr[6],
                      HlpStr,MainStr[7]);
    end else
    if (SigNal='Gd') or (Signal='FfA') then
    begin
      readln (OutFile,HlpStr,MainStr[1],HlpStr,MainStr[2],
                      HlpStr,MainStr[3]);
      readln (OutFile,HlpStr,MainStr[4],HlpStr,MainStr[5],
                      HlpStr,MainStr[6]);
    end else
    if (Signal='LrA') or (Signal='RrA') then
      readln (OutFile,HlpStr,MainStr[1],HlpStr,MainStr[2],
                      HlpStr,MainStr[3],HlpStr,MainStr[4])
    else
    if (Signal='It') then
    begin
      readln (OutFile,HlpStr,MainStr[1],HlpStr,MainStr[2],
                      HlpStr,MainStr[4],HlpStr,MainStr[5]);
      readln (OutFile,HlpStr,MainStr[3],HlpStr,MainStr[3],
                      HlpStr,MainStr[3],HlpStr,MainStr[3]);
    end else
    if (Signal='A') then
    begin
      readln (OutFile,HlpStr,MainStr[1],HlpStr,MainStr[2],
                      HlpStr,MainStr[3]);
      readln (OutFile,HlpStr,MainStr[4],HlpStr,MainStr[5]);
    end
    else Showmessage ('No output group could be selected');

    for k:=1 to NrOfitems do
    begin
      setlength (Variable[k],NrOfData);
      while Pos(' ',MainStr[k])>0 do
            delete(MainStr[k],Pos(' ',MainStr[k]),1);
      if (MainStr[k]='-') then
         DataMod.Variable[k,NrOfData-1]:=-1
      else
         DataMod.Variable[k,NrOfData-1]:=StringToFlo(MainStr[k]);
    end;

  end; {while NrOfData<Ny*Ns do}

  Closefile (OutFile);

end; {TDataMod.ReadOutputGroup}
{-----------------------------}



procedure TDataMod.ReadGroupData;
{-------------------------------}
var i, j, k   : integer;
    SeasonNr, NrOfSeasons : integer;
    SerialNr  : integer;
    GroupFile : textfile;
    HlpStr    : string[14];
    CodeStr   : string[7];
    ItemStr   : string[10];
begin
  AssignFile(GroupFile,FileName);
  {$I-} Reset(GroupFile); {$I+}
  NrOfLines:=0;
  while not eof (GroupFile) do
  begin
    readln (GroupFile);
    NrOfLines:=NrOflines+1;
  end;
  NrOfData := NrOfLines-6;
  reset (GroupFile);
  readln (GroupFile,CodeStr);
  reset (GroupFile);
  GroupNotRead:=false;
  if CodeStr='"Data f' then
  begin
     readln (GroupFile,GroupTitle);
     NrOfData:=NrOfData-1;
  end else
  begin
    GroupNotRead:=true;
    exit;
  end;
  readln (GroupFile,GrTitle1);
  readln (GroupFile,GrTitle2);
  read (GroupFile,HlpStr);
  readln (GroupFile,GroupMark);
  while Pos(' ',GroupMark)>0 do
        delete(GroupMark,Pos(' ',GroupMark),1);
  NrOfSeasons:=0;
  for k:=1 to 4 do
  begin
    readln (GroupFile,CodeStr,SeasonNr);
    if SeasonNr>NrOfSeasons then NrOfSeasons := SeasonNr;
  end;
  NrOfitems := 7;
  if (GroupMark='Zs') or (GroupMark='Dw') then
     NrOfitems := 1;
  if (GroupMark='Gd') then NrOfitems := 4;
  if (GroupMark='LrA') or (GroupMark='RrA') then
      NrOfitems := 4;
  if (GroupMark='It') then NrOfitems := 5;
  if (GroupMark='FfA') then NrOfitems := 5;
  if (GroupMark='A') then NrOfitems := 4;
  if (GroupMark='EaU ') or (GroupMark='Kr') then
      NrOfitems := 1;
  Ns := NrOfSeasons;
  Ny := NrOfData div Ns;
  Reset (GroupFile);
  readln (GroupFile,CodeStr);
  for k:=1 to 2 do readln (GroupFile);
  if CodeStr='"Data f' then readln (GroupFile);
  SerialNr:=0;
  for i:=1 to Ny do
  begin
    for j:=1 to Ns do
    begin
      for k:=1 to NrOfitems do
      begin
        if k=1 then read (GroupFile,HlpStr);
        read (GroupFile,ItemStr);
        setlength (Variable[k],SerialNr+1);
        while Pos(' ',ItemStr)>0 do
              delete(ItemStr,Pos(' ',ItemStr),1);
        if (ItemStr='-') or (ItemStr='n.a.') then
           Variable[k,SerialNr]:=0
        else
        begin
          Variable[k,SerialNr]:=StringToFlo(ItemStr);
          if (GroupMark='A') or (GroupMark='FfA') then
             Variable[k,SerialNr]:=100*Variable[k,SerialNr];
        end;
      end; {for k:=1 to NrOfitems do}
      readln(GroupFile);
      SerialNr:=SerialNr+1;
    end; {for j:=1 to Ns do}
  end; {for i:=1 to Ny do }
  reset (GroupFile);
  for k:=1 to NrOflines-1 do readln (GroupFile);
  for k:=1 to Ns do read (GroupFile,Ts[k]);
  closefile (GroupFile);
end; {TDataMod.ReadGroupData}
{---------------------------}



procedure TDataMod.WriteOutputLines(const DocName: string);
{---------------------------------------------------------}
var  OutFile: textfile;
     k: integer;
begin
  AssignFile(OutFile,DocName);
  {$I-} Rewrite(OutFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + DocName + '"could not be opened. It may be ' +
      'damaged or in use by another program.');
    exit;
  end;
  for k:=1 to NrOfLines do
    writeln(OutFile,FileLine[k-1]);
  Close(OutFile);
end; {TDataMod.WriteOutputLines}
{------------------------------}



procedure TDataMod.WriteOutputGroup(const DocName: string);
{---------------------------------------------------------}
var  GroupFile: textfile;
     k: byte;
begin
  AssignFile(GroupFile,DocName);
  {$I-} Rewrite(GroupFile); {$I+}
  if ioresult <> 0 then
  begin
    Showmessage('The file "' + DocName + '"could not be opened. It may be ' +
      'damaged or in use by another program.');
    exit;
  end;
  writeln (GroupFile,GroupTitle);
  for k:=1 to NrOfData+3 do
      writeln (GroupFile,FileLine[k-1]);
  writeln (GroupFile,'#');
  writeln (GroupFile,'Season durations');
  for k:=1 to Ns do write (GroupFile,Ts[k]:5:1,'  ');
  writeln (GroupFile,'months');
  Close(GroupFile);
end; {TDataMod.WriteOutputGroup}
{------------------------------}


{******************************************************************************
 Diverse routines
 ******************************************************************************}


function TDataMod.StringToFlo(Str : string) : double;
{---------------------------------------------------}
{accepts comma or point for decimal separator}
var
  OldDecSep : char;
begin
  try
    Result := StrToFloat(Str);
  except
    on EConvertError do begin
      OldDecSep := DecimalSeparator;
      {change to other decimal separator}
      if DecimalSeparator = ',' then
        DecimalSeparator := '.'
      else
        DecimalSeparator := ',';
      Result := StrToFloat(Str);
      {revert to original decimal separator}
      DecimalSeparator := OldDecSep;
    end;
  end;
end; {TSegRegForm.StringToFlo}
{----------------------------}



function TDataMod.Question(const Query: string): boolean;
begin Result := MessageDlg(Query, mtConfirmation, [mbYes, mbNo], 0) = mrYes;
end; {TDataMod.Question}



procedure TDataMod.DataModuleCreate(Sender: TObject);
begin Filename := '';
end;


end.

